@@ -1,11 +1,11 @@
 /*
  * AUTHOR: Jay Schmidgall <jay@spdbump.sungardss.com>
  *
- * $Id: list.c,v 1.4 1999/07/24 01:49:10 lister Exp $
+ * $Id: list.c,v 1.1 1999/08/04 21:24:14 danny Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <Xm/Form.h>
@@ -121,9 +121,9 @@ char *argv[];
     XtAppContext app;
     HighlightStruct *hs = XtNew(HighlightStruct);
     char *name =
-	"The matrix mimics the keyboard interaction\n\
-of XmList. Up and Down move the highlight up and down,\n\
-and Space toggles the selection.";
+	"The matrix mimics the keyboard interaction\n"
+	"of XmList. Up and Down move the highlight up and down,\n"
+	"and Space toggles the selection.";
     XmString xms;
 #ifdef USE_EDITRES
     extern void _XEditResCheckMessages();
@@ -175,6 +175,12 @@ and Space toggles the selection.";
     hs->mw = mw;
     
     XtRealizeWidget(toplevel);
+
+#if 1
+    XbaeMatrixEditCell(mw, 0, 0);
+    XbaeMatrixCommitEdit(mw, True); /* unmap it again */
+#endif
+
     XtAppMainLoop(app);
     /*NOTREACHED*/
     return 0;
