@@ -21,15 +21,16 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Actions.c,v 1.19 1999/07/27 09:10:00 lister Exp $
+ * $Id: Actions.c,v 1.8 2000/03/28 21:44:10 danny Exp $
  */
 
+#undef	VERBOSE_SLIDE
 /*
  * Actions.c created by Andrew Lister (7 August, 1995)
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <Xm/Xm.h>
@@ -63,15 +64,14 @@
 typedef struct {
     XbaeMatrixWidget mw;
     GC gc;
-    int row;
-    int column;
-    int startx;
-    int lastx;
-    int currentx;
-    int y, height;
-    short *columnWidths;
+    int row, column;
+    int startx, starty;
+    int lastx, lasty;
+    int currentx, currenty;
+    int x, width, y, height;
+    short *columnWidths, *rowHeights;
     Boolean grabbed;
-    Boolean haveVSB;
+    Boolean haveVSB, haveHSB;
 } XbaeMatrixResizeColumnStruct;
 
 typedef struct {
@@ -97,13 +97,14 @@ typedef struct {
     Boolean right;
 } XbaeMatrixScrollStruct;
 
-static int DoubleClick P((XbaeMatrixWidget, XEvent *, int, int));
-static void DrawSlideColumn P((XbaeMatrixWidget, int));
-static void SlideColumn P((Widget, XtPointer, XEvent *, Boolean *));
-static void PushButton P((Widget, XtPointer, XEvent *, Boolean *));
-static void updateScroll P((XtPointer));
-static void checkScrollValues P((Widget, XtPointer, XEvent *, Boolean *));
-static void callSelectCellAction P((XbaeMatrixWidget, XEvent *));
+static int DoubleClick(XbaeMatrixWidget, XEvent *, int, int);
+static void DrawSlideRow(XbaeMatrixWidget, int);
+static void DrawSlideColumn(XbaeMatrixWidget, int);
+static void SlideColumn(Widget, XtPointer, XEvent *, Boolean *);
+static void PushButton(Widget, XtPointer, XEvent *, Boolean *);
+static void updateScroll(XtPointer);
+static void checkScrollValues(Widget, XtPointer, XEvent *, Boolean *);
+static void callSelectCellAction(XbaeMatrixWidget, XEvent *);
 
 static int last_row = 0;
 static int last_column = 0;
@@ -115,11 +116,7 @@ static Boolean scrolling = False;
 
 /* ARGSUSED */
 void
-xbaeDefaultActionACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeDefaultActionACT(Widget w, XEvent *event, String *params, Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     int x, y;
@@ -163,14 +160,12 @@ Cardinal *nparams;
 
 	XtCallCallbackList((Widget)mw, mw->matrix.default_action_callback,
 			   (XtPointer)&call_data);
-      
+
     }
 }
 
 static void
-DrawSlideColumn(mw, x)
-XbaeMatrixWidget mw;
-int x;
+DrawSlideColumn(XbaeMatrixWidget mw, int x)
 {
 #ifdef DRAW_RESIZE_SHADOW
     /* These values derived through that age-old process
@@ -216,12 +211,12 @@ int x;
 		      LeftClip(mw)->core.height);
 #endif
 #ifdef DRAW_RESIZE_SHADOW
-	DRAW_SHADOW(display, win,
+	DRAW_SHADOW(display, win, "win 1",
 		    mw->matrix.resize_top_shadow_gc,
 		    mw->matrix.resize_bottom_shadow_gc,
 		    shadow_width, x, y, width, height, XmSHADOW_OUT);
 	if (XtIsManaged(LeftClip(mw)))
-	    DRAW_SHADOW(display, XtWindow(LeftClip(mw)),
+	    DRAW_SHADOW(display, XtWindow(LeftClip(mw)), "LeftClip",
 			mw->matrix.resize_top_shadow_gc,
 			mw->matrix.resize_bottom_shadow_gc,
 			shadow_width, x - COLUMN_LABEL_OFFSET(mw),
@@ -255,12 +250,12 @@ int x;
 	
 #endif
 #ifdef DRAW_RESIZE_SHADOW
-	DRAW_SHADOW(display, win,
+	DRAW_SHADOW(display, win, "win 2",
 		    mw->matrix.resize_top_shadow_gc,
 		    mw->matrix.resize_bottom_shadow_gc,
 		    shadow_width, x, y, width, height, XmSHADOW_OUT);
 	if (XtIsManaged(RightClip(mw)))
-	    DRAW_SHADOW(display, XtWindow(RightClip(mw)),
+	    DRAW_SHADOW(display, XtWindow(RightClip(mw)), "RightClip",
 			mw->matrix.resize_top_shadow_gc,
 			mw->matrix.resize_bottom_shadow_gc,
 			shadow_width,
@@ -269,7 +264,7 @@ int x;
 #endif
 	return;
     }
-    
+
     xbaeGetVisibleRows(mw, &top, &bottom);
     /*
      * we need all non-fixed rows, so add 1 to bottom
@@ -299,7 +294,7 @@ int x;
     XDrawLine(display, win, gc, adjusted_x, y, adjusted_x, y + height);
 #endif
 #ifdef DRAW_RESIZE_SHADOW
-    DRAW_SHADOW(display, win,
+    DRAW_SHADOW(display, win, "win 3",
 		mw->matrix.resize_top_shadow_gc,
 		mw->matrix.resize_bottom_shadow_gc,
 		shadow_width, adjusted_x, y, width, height, XmSHADOW_OUT);
@@ -322,7 +317,7 @@ int x;
 #endif
 #ifdef DRAW_RESIZE_SHADOW
 	if (XtIsManaged(TopClip(mw)))
-	    DRAW_SHADOW(display, XtWindow(TopClip(mw)),
+	    DRAW_SHADOW(display, XtWindow(TopClip(mw)), "TopClip",
 			mw->matrix.resize_top_shadow_gc,
 			mw->matrix.resize_bottom_shadow_gc,
 			shadow_width, adjusted_x,
@@ -331,7 +326,7 @@ int x;
 #endif
 	xbaeSetClipMask(mw, CLIP_NONE);			 
     }
-    
+
     /*
      * The trailingFixedRows
      */
@@ -356,7 +351,7 @@ int x;
 #endif
 #ifdef DRAW_RESIZE_SHADOW
 	if (XtIsManaged(BottomClip(mw)))
-	    DRAW_SHADOW(display, XtWindow(BottomClip(mw)),
+	    DRAW_SHADOW(display, XtWindow(BottomClip(mw)), "BottomClip",
 			mw->matrix.resize_top_shadow_gc,
 			mw->matrix.resize_bottom_shadow_gc,
 			shadow_width, adjusted_x,
@@ -388,7 +383,7 @@ int x;
 		  adjusted_x, y, adjusted_x, height);
 #endif
 #ifdef DRAW_RESIZE_SHADOW
-	DRAW_SHADOW(display, XtWindow(mw),
+	DRAW_SHADOW(display, XtWindow(mw), "win 4",
 		    mw->matrix.resize_top_shadow_gc,
 		    mw->matrix.resize_bottom_shadow_gc,
 		    shadow_width, x, y,
@@ -397,20 +392,15 @@ int x;
     }
 }
 
-
 static void
-SlideColumn(w, data, event, cont)
-Widget w;
-XtPointer data;
-XEvent *event;
-Boolean *cont;
+SlideColumn(Widget w, XtPointer data, XEvent *event, Boolean *cont)
 {
     XbaeMatrixResizeColumnStruct *rd = (XbaeMatrixResizeColumnStruct *)data;
     XMotionEvent *motionEvent;
     Boolean relayout = False;
     int numCharacters;
     int i;
-    
+
     if (event->type == ButtonRelease)
     {
 	DrawSlideColumn(rd->mw, rd->lastx);
@@ -483,7 +473,7 @@ Boolean *cont;
 	rd->currentx -= numCharacters * FONT_WIDTH(rd->mw);
 	relayout = True;
     }	
-    
+
     if (motionEvent->x - rd->currentx > FONT_WIDTH(rd->mw))
     {
 	/*
@@ -510,13 +500,386 @@ Boolean *cont;
     }
 }
 
+/* Derived from DrawSlideColumn */
+static void
+DrawSlideRow(XbaeMatrixWidget mw, int y)
+{
+#ifdef DRAW_RESIZE_SHADOW
+    /* These values derived through that age-old process
+     * of what looks good to me */
+#define SHADOW_WIDTH 2
+#define RESIZE_COLUMN_LINE_WIDTH 4
+    Dimension height = RESIZE_COLUMN_LINE_WIDTH;
+    Dimension shadow_width = SHADOW_WIDTH;
+#endif
+    Dimension width;
+    Window win;
+    Display *display = XtDisplay(mw);
+    int row = xbaeYtoRow(mw, y - ROW_LABEL_OFFSET(mw));
+    int left, right;
+    int adjusted_y;
+    int x;
+#ifdef DRAW_RESIZE_LINE
+    GC gc = mw->matrix.draw_gc;
+#endif
+    Boolean need_vert_dead_space_fill = NEED_VERT_DEAD_SPACE_FILL(mw);
+    unsigned int clip_reason;
+
+#ifdef	VERBOSE_SLIDE
+    fprintf(stderr, "DrawSlideRow(%d)\n", y);
+#endif
+
+    /*
+     * If the column being resized is a fixed one then we don't need to
+     * bother with the clip region
+     */
+    if (row < (int)mw->matrix.fixed_rows)
+    {
+	x = COLUMN_LABEL_OFFSET(mw);
+	width = VISIBLE_WIDTH(mw) + FIXED_COLUMN_WIDTH(mw) +
+	    TRAILING_FIXED_COLUMN_WIDTH(mw);
+	win = XtWindow (mw);
+
+	if (need_vert_dead_space_fill)
+	    width += VERT_DEAD_SPACE_HEIGHT(mw);
+
+#ifdef DRAW_RESIZE_LINE
+#ifdef	VERBOSE_SLIDE
+	fprintf(stderr, "XDrawLine(%d,%d,%d,%d)\n", x, y, x + width, y);
+#endif
+	XDrawLine(display, win, gc, x, y, x + width, y);
+	if (XtIsManaged(TopClip(mw))) {					/* Used to be left */
+#ifdef	VERBOSE_SLIDE
+	    fprintf(stderr, "XDrawLine(%d,%d,%d,%d)\n",
+		      0, y - ROW_LABEL_OFFSET(mw),
+		      TopClip(mw)->core.width, y - ROW_LABEL_OFFSET(mw));
+#endif
+	    XDrawLine(display, XtWindow(TopClip(mw)), gc,
+		      0, y - ROW_LABEL_OFFSET(mw),
+		      TopClip(mw)->core.width, y - ROW_LABEL_OFFSET(mw));
+	}
+#endif
+#ifdef DRAW_RESIZE_SHADOW
+	DRAW_SHADOW(display, win, "win 5",
+		    mw->matrix.resize_top_shadow_gc,
+		    mw->matrix.resize_bottom_shadow_gc,
+		    shadow_width, x, y, width, height, XmSHADOW_OUT);
+	if (XtIsManaged(TopClip(mw)))
+	    DRAW_SHADOW(display, XtWindow(TopClip(mw)), "TopClip",
+			mw->matrix.resize_top_shadow_gc,
+			mw->matrix.resize_bottom_shadow_gc,
+			shadow_width,
+			0, y - ROW_LABEL_OFFSET(mw),
+			TopClip(mw)->core.width, height, XmSHADOW_OUT);
+#endif
+	return;
+    }
+    /*
+     * Similarly for trailingFixedRows - beware going off the clip child
+     * here also
+     */
+    if (row >= TRAILING_VERT_ORIGIN(mw) ||
+	y >= (int)(ClipChild(mw)->core.y + ClipChild(mw)->core.height))
+    {
+	x = COLUMN_LABEL_OFFSET(mw);
+	width = VISIBLE_WIDTH(mw) + FIXED_COLUMN_WIDTH(mw) +
+	    TRAILING_FIXED_COLUMN_WIDTH(mw);
+	win = XtWindow(mw);
+
+	if (need_vert_dead_space_fill)
+	    width += VERT_DEAD_SPACE_HEIGHT(mw);
+
+#ifdef DRAW_RESIZE_LINE
+#ifdef	VERBOSE_SLIDE
+	fprintf(stderr, "XDrawLine(%d,%d,%d,%d)\n", x, y, x + width, y);
+#endif
+	XDrawLine(display, win, gc, x, y, x + width, y);
+	if (XtIsManaged(BottomClip(mw))) {			/* Used to be RightClip */
+#ifdef	VERBOSE_SLIDE
+	    fprintf(stderr, "XDrawLine(BottomClip %d,%d,%d,%d)\n",
+		0, y - TRAILING_FIXED_ROW_LABEL_OFFSET(mw),
+		BottomClip(mw)->core.width, y - TRAILING_FIXED_ROW_LABEL_OFFSET(mw));
+#endif
+	    XDrawLine(display, XtWindow(BottomClip(mw)), gc,
+		0, y - TRAILING_FIXED_ROW_LABEL_OFFSET(mw),
+		BottomClip(mw)->core.width, y - TRAILING_FIXED_ROW_LABEL_OFFSET(mw));
+	}
+#endif
+#ifdef DRAW_RESIZE_SHADOW
+	DRAW_SHADOW(display, win, "win 6",
+		    mw->matrix.resize_top_shadow_gc,
+		    mw->matrix.resize_bottom_shadow_gc,
+		    shadow_width, x, y, width, height, XmSHADOW_OUT);
+	if (XtIsManaged(BottomClip(mw)))
+	    DRAW_SHADOW(display, XtWindow(BottomClip(mw)), "BottopClip",
+			mw->matrix.resize_top_shadow_gc,
+			mw->matrix.resize_bottom_shadow_gc,
+			shadow_width,
+		       	0, y - TRAILING_FIXED_ROW_LABEL_OFFSET(mw),
+		       	BottomClip(mw)->core.width, height,
+			XmSHADOW_OUT);
+#endif
+	return;
+    }
+
+    xbaeGetVisibleColumns(mw, &left, &right);
+    /*
+     * we need all non-fixed columns, so add 1 to left to include the last one
+     * as the return values are inclusive
+    left += 1;
+     */
+    right += 1;
+
+    /*
+     * The area between left and right rows are the non fixed columns.
+     * They fall on the ClipChild
+     */
+    x = -mw->matrix.cell_shadow_thickness; /* relative to clip */
+
+    width = COLUMN_POSITION(mw, right) - COLUMN_POSITION(mw, left)
+	+ 2 * mw->matrix.cell_shadow_thickness;
+#ifdef	VERBOSE_SLIDE
+    fprintf(stderr, "Width %d = %d - %d + 2 * %d (left %d, right %d)\n",
+	width, COLUMN_POSITION(mw, right), COLUMN_POSITION(mw, left),
+	mw->matrix.cell_shadow_thickness, left, right);
+#endif
+
+    /*
+     * If we are on the clip, the y location is offset by the
+     * fixed row height, label offset and label height
+     */
+    adjusted_y = y - FIXED_ROW_LABEL_OFFSET(mw);
+
+#ifdef	VERBOSE_SLIDE
+    fprintf(stderr, "DrawSlideRow(%d) adjust %d\n", y, adjusted_y);
+#endif
+
+    win = XtWindow(ClipChild(mw));
+
+#ifdef DRAW_RESIZE_LINE
+#ifdef	VERBOSE_SLIDE
+    fprintf(stderr, "XDrawLine(%d,%d,%d,%d)\n", x, adjusted_y, x + width, adjusted_y);
+#endif
+    XDrawLine(display, win, gc, x, adjusted_y, x + width, adjusted_y);
+#endif
+#ifdef DRAW_RESIZE_SHADOW
+    DRAW_SHADOW(display, win, "win 7",
+		mw->matrix.resize_top_shadow_gc,
+		mw->matrix.resize_bottom_shadow_gc,
+		shadow_width, x, adjusted_y, width, height, XmSHADOW_OUT);
+#endif
+    /*
+     * Now draw the line (or shadow) on the non clipped region - that is
+     * the fixed and trailingFixed rows.  First, do the leading rows.
+     */
+    if (mw->matrix.fixed_columns)
+    {
+	x = COLUMN_LABEL_OFFSET(mw);
+	width = FIXED_COLUMN_WIDTH(mw) + 2 * mw->matrix.cell_shadow_thickness;
+	win = XtWindow(mw);
+	xbaeSetClipMask(mw, CLIP_FIXED_COLUMNS);
+
+#ifdef DRAW_RESIZE_LINE
+	if (XtIsManaged(LeftClip(mw))) {
+#ifdef	VERBOSE_SLIDE
+	    fprintf(stderr, "XDrawLine(%d,%d,%d,%d)\n",
+		-mw->matrix.cell_shadow_thickness, adjusted_y, height, adjusted_y);
+#endif
+	    XDrawLine(display, XtWindow(TopClip(mw)), gc,
+		-mw->matrix.cell_shadow_thickness, adjusted_y, height, adjusted_y);
+	}
+#endif
+#ifdef DRAW_RESIZE_SHADOW
+	if (XtIsManaged(LeftClip(mw)))
+	    DRAW_SHADOW(display, XtWindow(TopClip(mw)), "LeftClip",
+			mw->matrix.resize_top_shadow_gc,
+			mw->matrix.resize_bottom_shadow_gc,
+			shadow_width,
+			-mw->matrix.cell_shadow_thickness, adjusted_y,
+			width, height, XmSHADOW_OUT);
+#endif
+	xbaeSetClipMask(mw, CLIP_NONE);			 
+    }
+
+    /*
+     * The trailingFixedRows
+     */
+    if (mw->matrix.trailing_fixed_columns)
+    {
+	x = TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw);
+	width = TRAILING_FIXED_COLUMN_WIDTH(mw) +
+	    2 * mw->matrix.cell_shadow_thickness;
+
+	clip_reason = CLIP_TRAILING_FIXED_COLUMNS;
+	if (IS_LEADING_FIXED_ROW(mw, row))
+	    clip_reason |= CLIP_FIXED_ROWS;
+	else if (IS_TRAILING_FIXED_ROW(mw, row))
+	    clip_reason |= CLIP_TRAILING_FIXED_ROWS;
+	
+	xbaeSetClipMask(mw, clip_reason);
+
+#ifdef DRAW_RESIZE_LINE
+	if (XtIsManaged(RightClip(mw)))
+	    XDrawLine(display, XtWindow(RightClip(mw)), gc,
+		-mw->matrix.cell_shadow_thickness, adjusted_y, width, adjusted_y);
+#endif
+#ifdef DRAW_RESIZE_SHADOW
+	if (XtIsManaged(RightClip(mw)))
+	    DRAW_SHADOW(display, XtWindow(RightClip(mw)), "RightClip",
+			mw->matrix.resize_top_shadow_gc,
+			mw->matrix.resize_bottom_shadow_gc,
+			shadow_width,
+			-mw->matrix.cell_shadow_thickness, adjusted_y,
+			width, height, XmSHADOW_OUT);
+#endif
+	xbaeSetClipMask(mw, CLIP_NONE);
+    }
+
+    if ((NEED_HORIZ_FILL(mw) && (! HAS_ATTACHED_TRAILING_COLUMNS(mw))) ||
+	need_vert_dead_space_fill)
+    {
+	if (need_vert_dead_space_fill)
+	{
+	    x = UNATTACHED_TRAILING_COLUMNS_OFFSET(mw) -
+		mw->matrix.cell_shadow_thickness;
+	    width = 2 * mw->matrix.cell_shadow_thickness +
+		VERT_DEAD_SPACE_HEIGHT(mw);
+	}
+	else
+	{
+	    x = TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw) +
+		TRAILING_FIXED_COLUMN_WIDTH(mw);
+	    width = FILL_HORIZ_WIDTH(mw) - VERT_SB_SPACE(mw);
+	}
+
+#ifdef DRAW_RESIZE_LINE
+	XDrawLine(display, XtWindow(mw), gc,
+		  x, adjusted_y, width, adjusted_y);
+#endif
+#ifdef DRAW_RESIZE_SHADOW
+	DRAW_SHADOW(display, XtWindow(mw), "win 8",
+		    mw->matrix.resize_top_shadow_gc,
+		    mw->matrix.resize_bottom_shadow_gc,
+		    shadow_width, x, y,
+		    width, height, XmSHADOW_OUT);
+#endif
+    }
+}
+
+/* Derived from SlideColumn */
+static void
+SlideRow(Widget w, XtPointer data, XEvent *event, Boolean *cont)
+{
+    XbaeMatrixResizeColumnStruct *rd = (XbaeMatrixResizeColumnStruct *)data;
+    XMotionEvent *motionEvent;
+    Boolean relayout = False;
+    int numCharacters;
+    int i;
+
+    if (event->type == ButtonRelease)
+    {
+	DrawSlideRow(rd->mw, rd->lasty);
+	XUngrabPointer(XtDisplay(w), CurrentTime);
+	rd->grabbed = False;
+	/*
+	 * Remanage the HSB if we unmapped it earlier
+	 */
+	if (rd->haveHSB)
+	    XtManageChild(HorizScrollChild(rd->mw));
+
+	if (rd->mw->matrix.resize_row_callback)
+	{
+	    XbaeMatrixResizeRowCallbackStruct call_data;
+
+	    call_data.reason = XbaeResizeColumnReason;
+	    call_data.event = event;
+	    call_data.row = rd->row - 1;
+	    call_data.column = rd->column;
+	    call_data.which = rd->row - 1;
+	    call_data.rows = rd->mw->matrix.rows;
+	    call_data.row_heights  = rd->rowHeights;
+	    XtCallCallbackList ((Widget)rd->mw,
+				rd->mw->matrix.resize_row_callback,
+				(XtPointer)&call_data);	    
+	}
+
+	for (i = 0; i < rd->mw->matrix.rows; i++)
+	    if (rd->rowHeights[i] != rd->mw->matrix.row_heights[i])
+	    {
+		/* Make sure everything is handled correctly with SetValues */
+		XtVaSetValues((Widget)rd->mw, XmNrowHeights,
+			       rd->rowHeights, NULL);
+		break;
+	    }
+#if 0
+	/*
+	 * If maxColumnLengths are set and we have resized the column to
+	 * larger, reset the corresponding maxColumnLength
+	 */
+	if (rd->mw->matrix.column_max_lengths &&
+	    rd->columnWidths[rd->column - 1] >
+	    rd->mw->matrix.column_max_lengths[rd->column - 1])
+	    rd->mw->matrix.column_max_lengths[rd->column - 1] =
+		rd->columnWidths[rd->column - 1];
+#endif
+	XtFree((char *)rd->rowHeights);
+	return;
+    }
+
+    if (event->type != MotionNotify) /* Double check! */
+	return;
+
+    motionEvent = (XMotionEvent *)event;
+
+    if (rd->currenty - motionEvent->y > 4)	/* ??? FONT_HEIGHT is not right */
+    {
+	/* If we're only one character wide, we cannae get any smaller */
+	if (rd->rowHeights[rd->row - 1] == 0)
+	    return;	
+	/*
+	 * Moved left a full character - update the column widths and force
+	 * a redisplay
+	 */	
+	numCharacters = (rd->currenty - motionEvent->y);
+	if (numCharacters < 0)
+	    numCharacters = 0;
+
+	rd->rowHeights[rd->row - 1] -= numCharacters;
+	rd->currenty -= numCharacters;
+	relayout = True;
+    }
+
+    if (motionEvent->y - rd->currenty > 4)	/* ??? FONT_HEIGHT is not right */
+    {
+	/*
+	 * Moved right a full character - update the column widths and force
+	 * a redisplay
+	 */
+	numCharacters = (motionEvent->y - rd->currenty);
+	rd->rowHeights[rd->row - 1] += numCharacters;
+	rd->currenty += numCharacters;
+	relayout = True;
+    }
+
+    if (relayout)
+    {
+	/* Draw the marker line in the new location */
+	if (rd->lasty != rd->currenty)
+	{
+	    DrawSlideRow(rd->mw, rd->currenty);
+	    DrawSlideRow(rd->mw, rd->lasty);
+
+	    rd->lasty = rd->currenty;
+	}
+    }
+}
+
 /* ARGSUSED */
 void
-xbaeResizeColumnsACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeResizeRowsACT(Widget w,
+	XEvent *event,
+	String *params,
+	Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     int x, y;
@@ -524,7 +887,6 @@ Cardinal *nparams;
     int i;
     int row, column;
     CellType cell;
-    static Cursor cursor;
     XbaeMatrixResizeColumnStruct resizeData;
     XGCValues values;
     XtAppContext appcontext;
@@ -536,7 +898,8 @@ Cardinal *nparams;
 #define FUZZ_FACTOR	3
     int	fuzzy = FUZZ_FACTOR;
 #undef FUZZ_FACTOR
-    
+
+    fprintf(stderr, "xbaeResizeRowsACT()\n");
     /*
      * Get Matrix widget and make sure it is a Matrix subclass.
      * w could be Matrix, or the Clip or textField children of Matrix
@@ -553,21 +916,180 @@ Cardinal *nparams;
 	    NULL, 0);
 	return;
     }
-    
+
     /*
      * If we won't allow dynamic column resize, leave.
      */
-    if (!mw->matrix.allow_column_resize)
+    if ((!mw->matrix.allow_column_resize) && (!mw->matrix.allow_row_resize))
 	return;
-    
+
     if (!xbaeEventToXY(mw, event, &x, &y, &cell))
 	return;
-    
+
     eventx = x;
-    
+
     if (!xbaeXYToRowCol(mw, &x, &y, &row, &column, cell))
 	return;
 
+    /*
+     * Figure out if we're resizing a row height...
+     */
+    if ((int)mw->matrix.cell_shadow_thickness > fuzzy)
+	fuzzy = mw->matrix.cell_shadow_thickness;
+
+    if (y > fuzzy && SOME_ROW_HEIGHT(mw, row) - y > fuzzy)
+	return;
+
+    /* FIX ME currently not implemented yet */
+
+    fprintf(stderr, "xbaeResizeRowsACT() : do something\n");
+
+    /*
+     * Make it here and it's time to start the fun stuff!
+     */
+    /* Create the left / right cursor */
+    if (!mw->matrix.cursor)
+	mw->matrix.cursor = XCreateFontCursor(display, XC_sb_h_double_arrow);
+
+    /* Can't adjust the origin or should you be able to?? */
+    if (row == 0)
+	return;
+
+    /* Commit any edit in progress and unmap the text field -
+       it's just bad luck */
+    (*((XbaeMatrixWidgetClass)XtClass(mw))->matrix_class.commit_edit)
+	(mw, event, True);
+
+    /*
+     * Redraw the cell that had the text field in it or it might stay blank
+     */
+    xbaeDrawCell(mw, mw->matrix.current_row, mw->matrix.current_column);
+
+    /*
+     * Say goodbye to the Horizontal ScrollBar -> it only gets in the way!
+     */
+    if ((resizeData.haveHSB = XtIsManaged(HorizScrollChild(mw)) &&
+	   ((mw->matrix.scrollbar_placement == XmTOP_RIGHT) ||
+	    (mw->matrix.scrollbar_placement == XmBOTTOM_RIGHT))))
+        XtUnmanageChild(HorizScrollChild(mw));
+    /*
+     * Flush the commit events out to the server.  Otherwise, our changes
+     * to the GCs below have a bad effect.
+     */
+    XSync(display, False);
+
+    event_mask = PointerMotionMask | ButtonReleaseMask;
+    XtAddEventHandler(w, event_mask,
+		       True, (XtEventHandler)SlideRow,
+		       (XtPointer)&resizeData);
+
+    XGrabPointer(display, XtWindow(w), True, event_mask,
+		  GrabModeAsync, GrabModeAsync, XtWindow((Widget)mw),
+		  mw->matrix.cursor, CurrentTime);
+
+    /* Copy the rowHeights array */
+    resizeData.rowHeights =
+	 (short *)XtMalloc(mw->matrix.rows * sizeof(short));
+    for (i = 0; i < mw->matrix.rows; i++)
+	resizeData.rowHeights[i] = mw->matrix.row_heights[i];
+    resizeData.grabbed = True;
+    resizeData.mw = mw;
+    resizeData.row = row;
+    resizeData.starty = resizeData.currenty = resizeData.lasty =
+	event->xbutton.y;
+
+    gcmask = GCForeground | GCBackground | GCFunction;
+    values.function = GXxor;
+#ifdef DRAW_RESIZE_LINE
+    XGetGCValues(display, mw->matrix.draw_gc, gcmask, &save);
+    values.foreground = values.background = save.background;
+
+    XChangeGC(display, mw->matrix.draw_gc, gcmask, &values);
+#endif
+
+    DrawSlideRow(mw, resizeData.currenty);
+
+    appcontext = XtWidgetToApplicationContext(w);
+
+    while (resizeData.grabbed)
+	XtAppProcessEvent(appcontext, XtIMAll);
+
+    XtRemoveEventHandler(w, event_mask, True,
+			   (XtEventHandler)SlideRow,
+			   (XtPointer)&resizeData);
+
+#ifdef DRAW_RESIZE_LINE
+    XSetFunction(display, mw->matrix.draw_gc, GXcopy);
+#endif
+}
+
+/* ARGSUSED */
+void
+xbaeResizeColumnsACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
+{
+    XbaeMatrixWidget mw;
+    int x, y;
+    int eventx;
+    int i;
+    int row, column;
+    CellType cell;
+    XbaeMatrixResizeColumnStruct resizeData;
+    XGCValues values;
+    XtAppContext appcontext;
+#ifdef DRAW_RESIZE_LINE
+    XGCValues save;
+#endif
+    unsigned long gcmask, event_mask;
+    Display *display = XtDisplay(w);
+#define FUZZ_FACTOR	3
+    int	fuzzy = FUZZ_FACTOR;
+#undef FUZZ_FACTOR
+
+    /*
+     * Get Matrix widget and make sure it is a Matrix subclass.
+     * w could be Matrix, or the Clip or textField children of Matrix
+     */
+    if (XtIsSubclass(w, xbaeMatrixWidgetClass))
+	mw = (XbaeMatrixWidget)w;
+    else if (XtIsSubclass(XtParent(w), xbaeMatrixWidgetClass))
+	mw = (XbaeMatrixWidget)XtParent(w);
+    else {
+	XtAppWarningMsg(
+	    XtWidgetToApplicationContext(w),
+	    "resizeColumnsACT", "badWidget", "XbaeMatrix",
+	    "XbaeMatrix: Bad widget passed to ResizeColumns action",
+	    NULL, 0);
+	return;
+    }
+
+    /*
+     * If we won't allow dynamic column resize, leave.
+     */
+    if ((!mw->matrix.allow_column_resize) && (!mw->matrix.allow_row_resize))
+	return;
+
+    if (!xbaeEventToXY(mw, event, &x, &y, &cell))
+	return;
+
+    eventx = x;
+
+    if (!xbaeXYToRowCol(mw, &x, &y, &row, &column, cell))
+	return;
+
+    /*
+     * Figure out if we're resizing a row height...
+     */
+    if ((int)mw->matrix.cell_shadow_thickness > fuzzy)
+	fuzzy = mw->matrix.cell_shadow_thickness;
+
+    if (y <= fuzzy || SOME_ROW_HEIGHT(mw, row) - y <= fuzzy) {
+	xbaeResizeRowsACT(w, event, params, nparams);
+	return;
+    }
+
     /*
      * Calculate if the x and y of the middle button event is on
      * a column border.  Allow the width of the shadow to be the
@@ -576,29 +1098,29 @@ Cardinal *nparams;
      */
     if ((int)mw->matrix.cell_shadow_thickness > fuzzy)
 	fuzzy = mw->matrix.cell_shadow_thickness;
-    
+
     if (x > fuzzy && COLUMN_WIDTH(mw, column) - x > fuzzy)
 	return;
-    
+
     /*
      * Looks like we hit a column border, determine the column that is
      * intended to be resized
      */
     if ((COLUMN_WIDTH(mw, column) - x) <= fuzzy)
 	column++;
-    
+
     /* Can't adjust the origin or should you be able to?? */
     if (column == 0)
 	return;
-    
+
     /*
      * Make it here and it's time to start the fun stuff!
      */
-    
+
     /* Create the left / right cursor */
-    if (!cursor)
-	cursor = XCreateFontCursor(display, XC_sb_h_double_arrow);
-    
+    if (!mw->matrix.cursor)
+	mw->matrix.cursor = XCreateFontCursor(display, XC_sb_h_double_arrow);
+
     /* Commit any edit in progress and unmap the text field -
        it's just bad luck */
     (*((XbaeMatrixWidgetClass)XtClass(mw))->matrix_class.commit_edit)
@@ -621,16 +1143,16 @@ Cardinal *nparams;
      * to the GCs below have a bad effect.
      */
     XSync(display, False);
-    
+
     event_mask = PointerMotionMask | ButtonReleaseMask;
     XtAddEventHandler(w, event_mask,
 		       True, (XtEventHandler)SlideColumn,
 		       (XtPointer)&resizeData);
-    
+
     XGrabPointer(display, XtWindow(w), True, event_mask,
 		  GrabModeAsync, GrabModeAsync, XtWindow((Widget)mw),
-		  cursor, CurrentTime);
-    
+		  mw->matrix.cursor, CurrentTime);
+
     /* Copy the columnWidth array */
     resizeData.columnWidths =
 	 (short *)XtMalloc(mw->matrix.columns * sizeof(short));
@@ -650,14 +1172,14 @@ Cardinal *nparams;
 
     XChangeGC(display, mw->matrix.draw_gc, gcmask, &values);
 #endif
-    
+
     DrawSlideColumn(mw, resizeData.currentx);
 
     appcontext = XtWidgetToApplicationContext(w);
-    
+
     while (resizeData.grabbed)
 	XtAppProcessEvent(appcontext, XtIMAll);
-    
+
     XtRemoveEventHandler(w, event_mask, True,
 			   (XtEventHandler)SlideColumn,
 			   (XtPointer)&resizeData);
@@ -672,11 +1194,10 @@ Cardinal *nparams;
  */
 /* ARGSUSED */
 void
-xbaeProcessDragACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeProcessDragACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
 #if XmVersion > 1001
     XbaeMatrixWidget mw;
@@ -728,7 +1249,7 @@ Cardinal *nparams;
     else
 	call_data.string = mw->matrix.cells ?
 	    mw->matrix.cells[row][column] : "";
-    
+
     call_data.num_params = *nparams;
     call_data.params = params;
 
@@ -742,11 +1263,10 @@ Cardinal *nparams;
  */
 /* ARGSUSED */
 void
-xbaeEditCellACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeEditCellACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     int row, column;
@@ -1021,11 +1541,10 @@ Cardinal *nparams;
  */
 /* ARGSUSED */
 void
-xbaeCancelEditACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeCancelEditACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     Boolean unmap;
@@ -1089,11 +1608,10 @@ Cardinal *nparams;
  */
 /* ARGSUSED */
 void
-xbaeCommitEditACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeCommitEditACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     Boolean unmap;
@@ -1151,11 +1669,10 @@ Cardinal *nparams;
 }
 
 static int
-DoubleClick(mw, event, row, column)
-XbaeMatrixWidget mw;
-XEvent *event;
-int row;
-int column;
+DoubleClick(XbaeMatrixWidget mw,
+	XEvent *event,
+	int row,
+	int column)
 {
     /* A double click in this instance is two clicks in the
        same cell in a time period < double_click_interval */
@@ -1192,11 +1709,10 @@ int column;
 
 /*ARGSUSED*/
 static void
-PushButton(w, data, event, cont)
-Widget w;
-XtPointer data;
-XEvent *event;
-Boolean *cont;
+PushButton(Widget w,
+	XtPointer data,
+	XEvent *event,
+	Boolean *cont)
 {
     XbaeMatrixButtonPressedStruct *button =
 	 (XbaeMatrixButtonPressedStruct *)data;
@@ -1253,10 +1769,10 @@ Boolean *cont;
     motionEvent = (XMotionEvent *)event;
     x = motionEvent->x;
     y = motionEvent->y;
-    
+
     if (!xbaeEventToXY(button->mw, event, &x, &y, &cell))
 	return;
-    
+
     if (xbaeXYToRowCol(button->mw, &x, &y, &row, &column, cell))
 	/* Moved off the labels */
 	pressed = False;
@@ -1283,18 +1799,17 @@ Boolean *cont;
 
 /*ARGSUSED*/
 void
-xbaeHandleClick(w, data, event, cont)
-Widget w;
-XtPointer data;
-XEvent *event;
-Boolean *cont;
+xbaeHandleClick(Widget w,
+	XtPointer data,
+	XEvent *event,
+	Boolean *cont)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget)data;
     int x, y;
     CellType cell;
     int row, column;
     Boolean translation;
-    
+
     /* if we have a double click and a callback - break out! */
     if (event->type != ButtonPress && event->type != ButtonRelease)
 	return;
@@ -1348,7 +1863,7 @@ Boolean *cont;
 	XtAddEventHandler(TextChild(mw), event_mask,
 			   True, (XtEventHandler)PushButton,
 			   (XtPointer)&button);
-    
+
 	appcontext = XtWidgetToApplicationContext(w);
 
 	while (button.grabbed)
@@ -1386,11 +1901,10 @@ Boolean *cont;
 
 /* ARGSUSED */
 void
-xbaeSelectCellACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeSelectCellACT(Widget w,
+	XEvent *event,
+	String *params,
+	Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     int x, y;
@@ -1460,10 +1974,10 @@ Cardinal *nparams;
 	call_data.row = row;
 	call_data.column = column;
     }
-    
+
     last_selected_column = call_data.column;
     last_selected_row = call_data.row;
-    
+
     call_data.selected_cells = mw->matrix.selected_cells;
     call_data.cells = mw->matrix.cells;
     call_data.num_params = *nparams;
@@ -1476,11 +1990,10 @@ Cardinal *nparams;
 
 /* ARGSUSED */
 void
-xbaeTraverseNextACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeTraverseNextACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
 
@@ -1513,11 +2026,10 @@ Cardinal *nparams;
 
 /* ARGSUSED */
 void
-xbaeTraversePrevACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeTraversePrevACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
 
@@ -1549,13 +2061,11 @@ Cardinal *nparams;
 }
 
 static void
-callSelectCellAction(mw, event)
-XbaeMatrixWidget mw;
-XEvent *event;
+callSelectCellAction(XbaeMatrixWidget mw, XEvent *event)
 {
     XbaeMatrixSelectCellCallbackStruct call_data;
     Boolean old_scroll_select = mw->matrix.scroll_select;
-    
+
     mw->matrix.scroll_select = False;
 
     call_data.reason = XbaeSelectCellReason;
@@ -1567,7 +2077,7 @@ XEvent *event;
     call_data.num_params = 1;
     call_data.params = (char **)XtMalloc(sizeof(char *));
     call_data.params[0] = "extend";
-    
+
     XtCallCallbackList(
 	 (Widget)mw, mw->matrix.select_cell_callback,
 	 (XtPointer)&call_data);
@@ -1580,11 +2090,10 @@ XEvent *event;
 
 /*ARGSUSED*/
 static void
-checkScrollValues(w, data, event, cont)
-Widget w;
-XtPointer data;
-XEvent *event;
-Boolean *cont;
+checkScrollValues(Widget w,
+		XtPointer data,
+		XEvent *event,
+		Boolean *cont)
 {
     XbaeMatrixScrollStruct *ss = (XbaeMatrixScrollStruct *)data;
     XMotionEvent *motionEvent;
@@ -1728,8 +2237,7 @@ Boolean *cont;
 }
 
 static void
-updateScroll(data)
-XtPointer data;
+updateScroll(XtPointer data)
 {
     XbaeMatrixScrollStruct *ss = (XbaeMatrixScrollStruct *)data;
     Boolean callCallback = False;
@@ -1737,7 +2245,7 @@ XtPointer data;
 
     if (!scrolling)
 	return;
-    
+
     if (my_last_column != last_column || my_last_row != last_row)
 	callCallback = True;
 
@@ -1774,7 +2282,7 @@ XtPointer data;
 	 (ss->mw->matrix.selection_policy == XmMULTIPLE_SELECT ||
 	  ss->mw->matrix.selection_policy == XmEXTENDED_SELECT))
 	callSelectCellAction(ss->mw, ss->event);
-    
+
     /*
      * Flush the updates out to the server so we don't end up lagging
      * behind too far and end up with a million redraw requests.
@@ -1789,11 +2297,10 @@ XtPointer data;
 
 /* ARGSUSED */
 void
-xbaeHandleMotionACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaeHandleMotionACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     XbaeClipWidget cw;
@@ -1917,11 +2424,10 @@ Cardinal *nparams;
 
 /* ARGSUSED */
 void
-xbaePageDownACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaePageDownACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     char *down = "0";
@@ -1965,11 +2471,10 @@ Cardinal *nparams;
 
 /* ARGSUSED */
 void
-xbaePageUpACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+xbaePageUpACT(Widget w,
+		XEvent *event,
+		String *params,
+		Cardinal *nparams)
 {
     XbaeMatrixWidget mw;
     char *up = "0";
@@ -2011,4 +2516,3 @@ Cardinal *nparams;
 	XbaeMatrixEditCell((Widget)mw, VERT_ORIGIN(mw) +
 			   mw->matrix.fixed_rows, mw->matrix.current_column);
 }
-
