@@ -21,7 +21,7 @@
  *
  * ClipWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Clip.c,v 1.11 1999/07/24 02:03:48 lister Exp $
+ * $Id: Clip.c,v 1.3 2000/02/26 15:52:36 danny Exp $
  */
 
 /*
@@ -29,7 +29,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <X11/StringDefs.h>
@@ -38,7 +38,8 @@
 #include <Xbae/ClipP.h>
 
 static char defaultTranslations[] =
-    "<FocusIn>:			FocusIn()";
+	"<FocusIn>:		FocusIn()\n"
+;
 
 static XtResource resources[] = {
     { XmNexposeProc, XmCFunction, XtRFunction, sizeof(XtExposeProc),
@@ -52,26 +53,26 @@ static XtResource resources[] = {
 /*
  * Declaration of methods
  */
-static void ClassPartInitialize P((XbaeClipWidgetClass));
-static void Realize P((XbaeClipWidget, XtValueMask *,
-			XSetWindowAttributes *));
-static void Redisplay P((XbaeClipWidget, XEvent *, Region));
-static void Resize P((Widget));
-static void Redraw P((XbaeClipWidget));
+static void ClassPartInitialize(XbaeClipWidgetClass);
+static void Realize(XbaeClipWidget, XtValueMask *,
+			XSetWindowAttributes *);
+static void Redisplay(XbaeClipWidget, XEvent *, Region);
+static void Resize(Widget);
+static void Redraw(XbaeClipWidget);
 
 /*
  * Public convenience function
  */
-void XbaeClipRedraw P((Widget w));
+void XbaeClipRedraw(Widget w);
 
 /*
  * Clip actions
  */
-static void FocusInACT P((XbaeClipWidget, XEvent *, String *, Cardinal *));
+static void FocusInACT(XbaeClipWidget, XEvent *, String *, Cardinal *);
 
 static XtActionsRec actions[] =
 {
-    {"FocusIn", (XtActionProc)FocusInACT},
+	{ "FocusIn",	(XtActionProc)FocusInACT},
 };
 
 XbaeClipClassRec xbaeClipClassRec = {
@@ -133,8 +134,7 @@ WidgetClass xbaeClipWidgetClass = (WidgetClass) & xbaeClipClassRec;
 
 
 static void
-ClassPartInitialize(cwc)
-XbaeClipWidgetClass cwc;
+ClassPartInitialize(XbaeClipWidgetClass cwc)
 {
     register XbaeClipWidgetClass super =
 	(XbaeClipWidgetClass) cwc->core_class.superclass;
@@ -147,10 +147,9 @@ XbaeClipWidgetClass cwc;
 }
 
 static void
-Realize(cw, valueMask, attributes)
-XbaeClipWidget cw;
-XtValueMask *valueMask;
-XSetWindowAttributes *attributes;
+Realize(XbaeClipWidget cw,
+	XtValueMask *valueMask,
+	XSetWindowAttributes *attributes)
 {
     /*
      * Don't call our superclasses realize method, because Primitive sets
@@ -162,10 +161,9 @@ XSetWindowAttributes *attributes;
 
 /* ARGSUSED */
 static void
-Redisplay(cw, event, region)
-XbaeClipWidget cw;
-XEvent *event;
-Region region;
+Redisplay(XbaeClipWidget cw,
+	XEvent *event,
+	Region region)
 {
     if (cw->clip.expose_proc)
 	cw->clip.expose_proc((Widget)cw, event, region);
@@ -173,8 +171,7 @@ Region region;
 
 /* ARGSUSED */
 static void
-Resize(w)
-Widget w;
+Resize(Widget w)
 {
     /*
      * Xt will call the expose method when this method returns.
@@ -189,8 +186,7 @@ Widget w;
  */
 /* ARGSUSED */
 static void
-Redraw(cw)
-XbaeClipWidget cw;
+Redraw(XbaeClipWidget cw)
 {
     if (XtIsRealized((Widget)cw))
 	XClearArea(XtDisplay(cw), XtWindow(cw), 0, 0, 0, 0, True);
@@ -200,8 +196,7 @@ XbaeClipWidget cw;
  * Public interface to redraw method
  */
 void
-XbaeClipRedraw(w)
-Widget w;
+XbaeClipRedraw(Widget w)
 {
     /*
      * Make sure w is a Clip or a subclass
@@ -218,11 +213,10 @@ Widget w;
 
 /* ARGSUSED */
 static void
-FocusInACT(cw, event, params, nparams)
-XbaeClipWidget cw;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+FocusInACT(XbaeClipWidget cw,
+	XEvent *event,
+	String *params,
+	Cardinal *nparams)
 {
     if (event->xany.type != FocusIn || !event->xfocus.send_event)
 	return;
