@@ -22,7 +22,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Converters.c,v 1.12 1999/07/24 02:03:48 lister Exp $
+ * $Id: Converters.c,v 1.2 2000/02/26 15:52:36 danny Exp $
  */
 
 /*
@@ -35,7 +35,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <ctype.h>
@@ -52,16 +52,16 @@
 #define tolower(c)      ((c) - 'A' + 'a')
 #endif
 
-static Boolean StringsAreEqual P((String, String, int));
+static Boolean StringsAreEqual(String, String, int);
 
 /* ARGSUSED */
 Boolean
-CvtStringToStringArray(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToStringArray(Display *dpy,
+		XrmValuePtr args,
+		Cardinal *num_args,
+		XrmValuePtr from,
+		XrmValuePtr to,
+		XtPointer *data)
 {
     static String *array;
     String start = from->addr;
@@ -207,12 +207,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-StringArrayDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+StringArrayDestructor(XtAppContext app,
+			XrmValuePtr to,
+			XtPointer converter_data,
+			XrmValuePtr args,
+			Cardinal *num_args)
 {
     String *array = *(String **) to->addr;
     String *entry;
@@ -228,12 +227,12 @@ Cardinal *num_args;
 
 /* ARGSUSED */
 Boolean
-CvtStringToCellTable(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToCellTable(Display *dpy,
+		XrmValuePtr args,
+		Cardinal *num_args,
+		XrmValuePtr from,
+		XrmValuePtr to,
+		XtPointer *data)
 {
     static String **array;
     String start = from->addr;
@@ -324,12 +323,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-StringCellDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+StringCellDestructor(XtAppContext app,
+		XrmValuePtr to,
+		XtPointer converter_data,
+		XrmValuePtr args,
+		Cardinal *num_args)
 {
     String **array = *(String ***) to->addr;
     String **entry, *row;
@@ -353,12 +351,12 @@ Cardinal *num_args;
  */
 /* ARGSUSED */
 Boolean
-CvtStringToWidthArray(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToWidthArray(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static short *array;
     String start = from->addr;
@@ -431,12 +429,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-WidthArrayDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+WidthArrayDestructor(XtAppContext app,
+		XrmValuePtr to,
+		XtPointer converter_data,
+		XrmValuePtr args,
+		Cardinal *num_args)
 {
     short *array = *(short **) to->addr;
 
@@ -449,12 +446,12 @@ Cardinal *num_args;
  */
 /* ARGSUSED */
 Boolean
-CvtStringToMaxLengthArray(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToMaxLengthArray(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static int *array;
     String start = from->addr;
@@ -528,12 +525,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-MaxLengthArrayDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+MaxLengthArrayDestructor(XtAppContext app,
+			XrmValuePtr to,
+			XtPointer converter_data,
+			XrmValuePtr args,
+			Cardinal *num_args)
 {
     int *array = *(int **) to->addr;
 
@@ -546,10 +542,7 @@ Cardinal *num_args;
  * Used by StringToAlignmentArray converter.
  */
 static Boolean
-StringsAreEqual(in, test, length)
-String in;
-String test;
-int length;
+StringsAreEqual(String in, String test, int length)
 {
     int i;
 
@@ -583,12 +576,12 @@ int length;
  */
 /* ARGSUSED */
 Boolean
-CvtStringToBooleanArray(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToBooleanArray(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static Boolean *array;
     String start = from->addr;
@@ -681,12 +674,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-BooleanArrayDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+BooleanArrayDestructor(XtAppContext app,
+			XrmValuePtr to,
+			XtPointer converter_data,
+			XrmValuePtr args,
+			Cardinal *num_args)
 {
     Boolean *array = *(Boolean **) to->addr;
 
@@ -700,12 +692,12 @@ Cardinal *num_args;
  */
 /* ARGSUSED */
 Boolean
-CvtStringToAlignmentArray(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToAlignmentArray(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static unsigned char *array;
     String start = from->addr;
@@ -803,12 +795,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-AlignmentArrayDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+AlignmentArrayDestructor(XtAppContext app,
+			XrmValuePtr to,
+			XtPointer converter_data,
+			XrmValuePtr args,
+			Cardinal *num_args)
 {
     unsigned char *array = *(unsigned char **) to->addr;
 
@@ -821,12 +812,12 @@ Cardinal *num_args;
  */
 /* ARGSUSED */
 Boolean
-CvtStringToPixelTable(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToPixelTable(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static Pixel **array, *row;
     String start = from->addr;
@@ -957,12 +948,11 @@ XtPointer *data;
  */
 /* ARGSUSED */
 void
-PixelTableDestructor(app, to, converter_data, args, num_args)
-XtAppContext app;
-XrmValuePtr to;
-XtPointer converter_data;
-XrmValuePtr args;
-Cardinal *num_args;
+PixelTableDestructor(XtAppContext app,
+			XrmValuePtr to,
+			XtPointer converter_data,
+			XrmValuePtr args,
+			Cardinal *num_args)
 {
     Pixel **array = *(Pixel ***) to->addr;
     Pixel **col;
@@ -977,12 +967,12 @@ Cardinal *num_args;
 
 /* ARGSUSED */
 Boolean
-CvtStringToGridType(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToGridType(Display *dpy,
+		XrmValuePtr args,
+		Cardinal *num_args,
+		XrmValuePtr from,
+		XrmValuePtr to,
+		XtPointer *data)
 {
     static unsigned char grid_type;
     String start = from->addr;
@@ -1060,15 +1050,16 @@ Boolean
 #ifdef __VMS
 /* According to Barry Stone VMS only allows function names with a
    maximum length of 31 characters */
-CvtStringToMatrixScrollBarDisp(dpy, args, num_args, from, to, data)
+CvtStringToMatrixScrollBarDisp
 #else
-CvtStringToMatrixScrollBarDisplayPolicy(dpy, args, num_args, from, to, data)
+CvtStringToMatrixScrollBarDisplayPolicy
 #endif
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+	(Display *dpy,
+	XrmValuePtr args,
+	Cardinal *num_args,
+	XrmValuePtr from,
+	XrmValuePtr to,
+	XtPointer *data)
 {
     static unsigned char display_policy;
     String start = from->addr;
