@@ -20,11 +20,11 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Draw.c,v 1.18 1999/07/24 02:03:48 lister Exp $
+ * $Id: Draw.c,v 1.6 2000/02/29 19:29:23 danny Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <Xm/Xm.h>
@@ -38,35 +38,31 @@
 #include <Xbae/Draw.h>
 #include <stdlib.h>
 
-static void xbaeDrawCellString P((XbaeMatrixWidget, int, int, int, int,
-				  String, Pixel, Pixel));
-#if CELL_WIDGETS
-static void xbaeDrawCellWidget P((XbaeMatrixWidget, int, int, int, int,
-				  Widget, Pixel, Pixel));
-#endif
-static void xbaeDrawCellPixmap P((XbaeMatrixWidget, int, int, int, int,
+static void xbaeDrawCellString(XbaeMatrixWidget, int, int, int, int,
+				  String, Pixel, Pixel);
+static void xbaeDrawCellWidget(XbaeMatrixWidget, int, int, int, int,
+				  Widget, Pixel, Pixel);
+static void xbaeDrawCellPixmap(XbaeMatrixWidget, int, int, int, int,
 				  Pixmap, Pixmap, int, int, Pixel,
-				  Pixel, int));
+				  Pixel, int);
 
 /*
  * Draw a fixed or non-fixed cell. The coordinates are calculated relative
  * to the correct window and pixmap is copied to that window.
  */
 static void
-xbaeDrawCellPixmap(mw, row, column, x, y, pixmap, mask, width, height, bg,
-		   fg, depth)
-XbaeMatrixWidget mw;
-int row;
-int column;
-int x;
-int y;
-Pixmap pixmap;
-Pixmap mask;
-int width;
-int height;
-Pixel bg;
-Pixel fg;
-int depth;
+xbaeDrawCellPixmap(XbaeMatrixWidget mw,
+		int row,
+		int column,
+		int x,
+		int y,
+		Pixmap pixmap,
+		Pixmap mask,
+		int width,
+		int height,
+		Pixel bg,
+		Pixel fg,
+		int depth)
 {
     int src_x = 0, src_y, dest_x, dest_y;
     int copy_width, copy_height;
@@ -234,11 +230,14 @@ int depth;
  * to the correct window and the cell is drawn in that window.
  */
 static void
-xbaeDrawCellString(mw, row, column, x, y, string, bg, fg)
-XbaeMatrixWidget mw;
-int row, column;
-String string;
-Pixel bg, fg;
+xbaeDrawCellString(XbaeMatrixWidget mw,
+		int row,
+		int column,
+		int x,
+		int y,
+		String string,
+		Pixel bg,
+		Pixel fg)
 {
     GC gc;
     Widget w;
@@ -346,24 +345,30 @@ Pixel bg, fg;
     xbaeDrawCellShadow(mw, win, row, column, x, y, COLUMN_WIDTH(mw, column),
 		       ROW_HEIGHT(mw), False, False, False);
 }
-#if CELL_WIDGETS
+
 /*
  * Draw a user defined widget in the cell
  */
 static void
-xbaeDrawCellWidget(mw, row, column, x, y, widget, bg, fg)
-XbaeMatrixWidget mw;
-int row, column;
-int x, y;
-Widget widget;
-Pixel bg, fg;
+xbaeDrawCellWidget(XbaeMatrixWidget mw,
+		int row,
+		int column,
+		int x,
+		int y,
+		Widget widget,
+		Pixel bg,
+		Pixel fg)
 {
     GC gc;
     Widget w;
     Window win = xbaeGetCellWindow(mw, &w, row, column);
 
+    Boolean clipped = False;	/* To get 4.7 to compile. */
+
     if (!win)
 	return;
+    if (! XtIsManaged(widget))
+	return;
 
     gc = mw->matrix.draw_gc;
     XSetForeground(XtDisplay(mw), gc, bg);
@@ -374,15 +379,12 @@ Pixel bg, fg;
      * Draw the widget in the cell.
      */
     XtMoveWidget(widget,
-		 x + mw->matrix.cell_shadow_thickness +
-		 mw->matrix.cell_highlight_thickness,
-		 y + mw->matrix.cell_shadow_thickness +
-		 mw->matrix.cell_highlight_thickness);
+		 x + mw->matrix.cell_shadow_thickness + mw->matrix.cell_highlight_thickness,
+		 y + mw->matrix.cell_shadow_thickness + mw->matrix.cell_highlight_thickness);
 
     xbaeDrawCellShadow(mw, win, row, column, x, y, COLUMN_WIDTH(mw, column),
 		       ROW_HEIGHT(mw), False, clipped, False);
 }
-#endif
 
 /*
  * Width in pixels of a character in a given font
@@ -400,29 +402,20 @@ Pixel bg, fg;
  * use a GC clip_mask, so we clip by characters. This complicates the code.
  */
 void
-#if NeedFunctionPrototypes
-xbaeDrawString(XbaeMatrixWidget mw, Window win, GC gc, String string,
-	       int length, int x, int y, int maxlen, unsigned char alignment,
-	       Boolean highlight, Boolean bold, Boolean rowLabel,
-	       Boolean colLabel, Pixel color)
-#else
-xbaeDrawString(mw, win, gc, string, length, x, y, maxlen, alignment,
-	       highlight, bold, rowLabel, colLabel, color)
-XbaeMatrixWidget mw;
-Window win;
-GC gc;
-String string;
-int length;
-int x;
-int y;
-int maxlen;
-unsigned char alignment;
-Boolean highlight;
-Boolean bold;
-Boolean rowLabel;
-Boolean colLabel;
-Pixel color;
-#endif
+xbaeDrawString(XbaeMatrixWidget mw,
+		Window win,
+		GC gc,
+		String string,
+		int length,
+		int x,
+		int y,
+		int maxlen,
+		unsigned char alignment,
+		Boolean highlight,
+		Boolean bold,
+		Boolean rowLabel,
+		Boolean colLabel,
+		Pixel color)
 {
     int start, width, maxwidth;
     XFontStruct	*font_struct;
@@ -896,10 +889,11 @@ Pixel color;
 }
 
 void
-xbaeComputeCellColors(mw, row, column, fg, bg)
-XbaeMatrixWidget mw;
-int row, column;
-Pixel *fg, *bg;
+xbaeComputeCellColors(XbaeMatrixWidget mw,
+			int row,
+			int column,
+			Pixel *fg,
+			Pixel *bg)
 {
     Boolean alt = mw->matrix.alt_row_count ?
 	(row / mw->matrix.alt_row_count) % 2 : False;
@@ -907,14 +901,16 @@ Pixel *fg, *bg;
     /*
      * Compute the background and foreground colours of the cell
      */
-    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column])
-	if (mw->matrix.reverse_select)
+    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column]) {
+	if (mw->matrix.reverse_select) {
 	    if (mw->matrix.colors)
 		*bg = mw->matrix.colors[row][column];
 	    else
 		*bg = mw->manager.foreground;
+        }
 	else
 	    *bg = mw->matrix.selected_background;
+    }
     else if (mw->matrix.cell_background &&
 	     mw->matrix.cell_background[row][column] !=
 	     mw->core.background_pixel)
@@ -927,14 +923,16 @@ Pixel *fg, *bg;
 	    *bg = mw->matrix.even_row_background;
     }
 
-    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column])
-	if (mw->matrix.reverse_select)
+    if (mw->matrix.selected_cells && mw->matrix.selected_cells[row][column]) {
+	if (mw->matrix.reverse_select) {
 	    if (mw->matrix.cell_background)
 		*fg = mw->matrix.cell_background[row][column];
 	    else
 		*fg = mw->core.background_pixel;
+        }
 	else
 	    *fg = mw->matrix.selected_foreground;
+    }
     else if (mw->matrix.colors)
 	*fg = mw->matrix.colors[row][column];
     else
@@ -942,9 +940,7 @@ Pixel *fg, *bg;
 }
 
 void
-xbaeDrawCell(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeDrawCell(XbaeMatrixWidget mw, int row, int column)
 {
     Pixel bg, fg;
     String string;
@@ -962,13 +958,10 @@ int row, column;
 
     xbaeComputeCellColors(mw, row, column, &fg, &bg);
 
-#if CELL_WIDGETS
-    if (mw->matrix.cell_widgets[row][column])
+    if (mw->matrix.cell_widgets && mw->matrix.cell_widgets[row][column])
 	xbaeDrawCellWidget(mw, row, column, x, y,
 			   mw->matrix.cell_widgets[row][column], bg, fg);
     else
-#endif
-
 	if (!mw->matrix.draw_cell_callback)
 	{
 	    if (row < mw->matrix.rows && column < mw->matrix.columns)
@@ -1001,17 +994,17 @@ int row, column;
 }
 
 XbaeCellType
-xbaeGetDrawCellValue(mw, row, column, string, pixmap, mask, width,
-		     height, bg, fg, depth)
-XbaeMatrixWidget mw;
-int row;
-int column;
-String *string;
-Pixmap *pixmap;
-Pixmap *mask;
-int *width, *height;
-Pixel *bg, *fg;
-int *depth;
+xbaeGetDrawCellValue(XbaeMatrixWidget mw,
+			int row,
+			int column,
+			String *string,
+			Pixmap *pixmap,
+			Pixmap *mask,
+			int *width,
+			int *height,
+			Pixel *bg,
+			Pixel  *fg,
+			int *depth)
 {
     XbaeMatrixDrawCellCallbackStruct call_data;
 
@@ -1108,14 +1101,7 @@ int *depth;
  * fixed and non-fixed columns.
  */
 void
-#if NeedFunctionPrototypes
 xbaeDrawColumnLabel(XbaeMatrixWidget mw, int column, Boolean pressed)
-#else
-xbaeDrawColumnLabel(mw, column, pressed)
-XbaeMatrixWidget mw;
-int column;
-Boolean pressed;
-#endif
 {
     String label;
     int labelX, labelY;
@@ -1218,14 +1204,7 @@ Boolean pressed;
  * non-fixed rows.
  */
 void
-#if NeedFunctionPrototypes
 xbaeDrawRowLabel(XbaeMatrixWidget mw, int row, Boolean pressed)
-#else
-xbaeDrawRowLabel(mw, row, pressed)
-XbaeMatrixWidget mw;
-int row;
-Boolean pressed;
-#endif
 {
     int y;
     GC gc;
