@@ -21,13 +21,13 @@
  *
  * Author: Andrew Lister
  *
- * $Id: Input.c,v 1.3 1999/07/27 09:10:01 lister Exp $
+ * $Id: Input.c,v 1.4 2000/02/29 19:29:23 danny Exp $
  */
 
 /*#define DEBUG*/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 /* For memmove/bcopy */
@@ -45,7 +45,7 @@
 #define XBAE_NEED_BCOPY
 #endif
 
-#if !defined(XBAE_NEED_BCOPY) || defined(SVR4) || defined(VMS) || defined(__EMX__)
+#if !defined(XBAE_NEED_BCOPY) || defined(SVR4) || defined(VMS)
 #define BCOPY(src, dest, n)     memmove((void *)(dest), (void *)(src), (n))
 #else
 #define BCOPY(src, dest, n)     bcopy((void *)(src), (void *)(dest), (n))
@@ -76,22 +76,22 @@
 /* Declaration of Actions */
 
 /* ARGSUSED */
-static void ToggleOverwriteACT P((Widget, XEvent *,
-				  String *, Cardinal *nparams));
+static void ToggleOverwriteACT(Widget, XEvent *,
+				  String *, Cardinal *nparams);
 
-static void validate P((Widget, XtPointer, XtPointer));
+static void validate(Widget, XtPointer, XtPointer);
 
 /* Declaration of Methods */
-static void Initialize P((Widget, Widget, ArgList, Cardinal *));
-static Boolean SetValues P((Widget, Widget, Widget, ArgList, Cardinal *));
-static void Destroy P((Widget));
+static void Initialize(Widget, Widget, ArgList, Cardinal *);
+static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
+static void Destroy(Widget);
 
 /* Utility routines */
-static void parsePattern P((XbaeInputWidget, String));
-static int match P((XbaeInputWidget, char *, int));
+static void parsePattern(XbaeInputWidget, String);
+static int match(XbaeInputWidget, char *, int);
 
 /* Internal callbacks */
-static void checkInput P((Widget, XtPointer, XtPointer));
+static void checkInput(Widget, XtPointer, XtPointer);
 
 static char defaultTranslations[] =
 "<Key>osfInsert : ToggleOverwrite()\n\
@@ -197,10 +197,7 @@ WidgetClass xbaeInputWidgetClass = (WidgetClass) &xbaeInputClassRec;
 
 /*ARGSUSED*/
 static void
-validate(w, cd, cb)
-Widget w;
-XtPointer cd;
-XtPointer cb;
+validate(Widget w, XtPointer cd, XtPointer cb)
 {
     XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *)cb;
     XmTextVerifyCallbackStruct *vcbs = (XmTextVerifyCallbackStruct *)cb;
@@ -296,11 +293,7 @@ XtPointer cb;
 
 /* ARGSUSED */
 static void
-ToggleOverwriteACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+ToggleOverwriteACT(Widget w, XEvent *event, String *params, Cardinal *nparams)
 {
     XbaeInputWidget iw = (XbaeInputWidget)w;
 
@@ -318,10 +311,7 @@ Cardinal *nparams;
 
 
 static void
-Initialize(request, new, args, num_args)
-Widget request, new;
-ArgList args;
-Cardinal *num_args;
+Initialize(Widget request, Widget new, ArgList args, Cardinal *num_args)
 {
     XbaeInputWidget iw = (XbaeInputWidget)new;
 
@@ -362,10 +352,8 @@ Cardinal *num_args;
 }
 
 static Boolean
-SetValues(current, request, new, args, num_args)
-Widget current, request, new;
-ArgList args;
-Cardinal *num_args;
+SetValues(Widget current, Widget request, Widget new,
+	  ArgList args, Cardinal *num_args)
 {
     XbaeInputWidget c = (XbaeInputWidget)current;
     XbaeInputWidget n = (XbaeInputWidget)new;
@@ -423,8 +411,7 @@ Cardinal *num_args;
 }
 
 static void
-Destroy(w)
-Widget w;
+Destroy(Widget w)
 {
     XbaeInputWidget iw = (XbaeInputWidget)w;
 
@@ -433,9 +420,7 @@ Widget w;
 }
 
 static void
-parsePattern(iw, pattern)
-XbaeInputWidget iw;
-String pattern;
+parsePattern(XbaeInputWidget iw, String pattern)
 {
     char *ptr = pattern;
     Boolean optional = False;
@@ -514,10 +499,7 @@ String pattern;
 }
 
 static int
-match(iw, string, exact)
-XbaeInputWidget iw;
-char *string;
-int exact;
+match(XbaeInputWidget iw, char *string, int exact)
 {
     char *tptr, *sptr, *ntptr;
     char *lastopt = NULL;	/* last optional block */
@@ -887,10 +869,7 @@ int exact;
 
 /*ARGSUSED*/
 static void
-checkInput(w, cd, cb)
-Widget w;
-XtPointer cd;
-XtPointer cb;
+checkInput(Widget w, XtPointer cd, XtPointer cb)
 {
     XbaeInputWidget iw = (XbaeInputWidget)w;
     XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *)cb;
@@ -909,6 +888,19 @@ XtPointer cb;
     if (cbs->startPos < cbs->currInsert || cbs->text->length == 0 ||
 	!cbs->text->ptr || *cbs->text->ptr == 0)
     {
+#if 1
+	/*
+	 * Fix by Anphong Nguyen <anphongn@davsys.com> :
+	 * codes added to move the cursor back one position, instead of deleting
+	 * the character because in overstriking mode, user uses backspace key
+	 * to move the cursor to overwrite the text, not to delete the character
+	 * then inserting a new value */
+	if (iw->input.overwrite_mode == True) {
+		fprintf(stderr, "Yow\n");
+		XmTextSetInsertionPosition((Widget)iw,(cbs->currInsert - 1));
+		cbs->doit = False;
+	}
+#endif
 	/*
 	 * Allow a backspace or delete to do it's bit.
 	 */	
@@ -1043,11 +1035,7 @@ XtPointer cb;
 }
 
 Widget
-XbaeCreateInput(parent, name, args, ac) 
-Widget parent;
-String name;
-ArgList args;
-Cardinal ac;
+XbaeCreateInput(Widget parent, String name, ArgList args, Cardinal ac) 
 { 
     return XtCreateWidget(name, xbaeInputWidgetClass, parent, args, ac); 
 } 
