@@ -20,7 +20,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: ScrollMgr.c,v 1.16 1999/07/24 02:03:48 lister Exp $
+ * $Id: ScrollMgr.c,v 1.7 2000/03/28 21:44:11 danny Exp $
  */
 
 /*
@@ -29,7 +29,7 @@
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <Xm/Xm.h>
@@ -66,7 +66,7 @@
  * Create and initialize a ScrollMgr
  */
 SmScrollMgr
-xbaeSmCreateScrollMgr()
+xbaeSmCreateScrollMgr(void)
 {
     SmScrollMgr scrollMgr = XtNew(SmScrollMgrRec);
 
@@ -83,8 +83,7 @@ xbaeSmCreateScrollMgr()
  * Destroy a ScrollMgr, including any queued scrolls
  */
 void
-xbaeSmDestroyScrollMgr(scrollMgr)
-SmScrollMgr scrollMgr;
+xbaeSmDestroyScrollMgr(SmScrollMgr scrollMgr)
 {
     if (scrollMgr->scroll_queue)
     {
@@ -107,10 +106,7 @@ SmScrollMgr scrollMgr;
  * Record a new scroll request in the ScrollMgr
  */
 void
-xbaeSmAddScroll(scrollMgr, delta_x, delta_y)
-SmScrollMgr scrollMgr;
-int delta_x;
-int delta_y;
+xbaeSmAddScroll(SmScrollMgr scrollMgr, int delta_x, int delta_y)
 {
     SmScrollNode node = XtNew(SmScrollNodeRec);
 
@@ -145,8 +141,7 @@ int delta_y;
  * Remove a scroll from the ScrollMgr queue
  */
 void
-xbaeSmRemoveScroll(scrollMgr)
-SmScrollMgr scrollMgr;
+xbaeSmRemoveScroll(SmScrollMgr scrollMgr)
 {
     if (scrollMgr->scroll_count)
     {
@@ -176,9 +171,7 @@ SmScrollMgr scrollMgr;
  * Handle an expose event
  */
 void
-xbaeSmScrollEvent(scrollMgr, event)
-SmScrollMgr scrollMgr;
-XEvent *event;
+xbaeSmScrollEvent(SmScrollMgr scrollMgr, XEvent *event)
 {
     switch (event->type)
     {
@@ -242,10 +235,7 @@ XEvent *event;
  */
 /* ARGSUSED */
 void
-xbaeScrollVertCB(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XmScrollBarCallbackStruct *call_data;
+xbaeScrollVertCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct *call_data)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
     Rectangle fixed, nonfixed;
@@ -258,7 +248,7 @@ XmScrollBarCallbackStruct *call_data;
     int trailing_fixed_column_width = TRAILING_FIXED_COLUMN_WIDTH(mw);
     int row_label_width = ROW_LABEL_WIDTH(mw);
     int fixed_row_label_offset = FIXED_ROW_LABEL_OFFSET(mw);
-    
+
     /*
      * Not managed yet
      */
@@ -303,6 +293,27 @@ XmScrollBarCallbackStruct *call_data;
 		     TextChild(mw)->core.x, TextChild(mw)->core.y +
 		     (VERT_ORIGIN(mw) - call_data->value) * row_height);
 
+    /* Cell widgets that are in the vertical scrolling region 
+     * need to scroll along as well... Linas */
+    if (mw->matrix.cell_widgets) {
+       int roe, kol;
+
+       for (roe=mw->matrix.fixed_rows;
+            roe < TRAILING_VERT_ORIGIN(mw); roe ++) {
+          for (kol=0; kol < (mw->matrix.columns); kol++) { 
+             Widget w = mw->matrix.cell_widgets[roe][kol];
+             if (w) {
+                if (XtIsManaged (w)) {
+                   int x = w->core.x;
+                   int y = w->core.y;
+                   y += (VERT_ORIGIN(mw) - call_data->value) * row_height;
+                   XtMoveWidget(w, x, y);
+                }
+             }
+          }
+       }
+    }
+
     /*
      * Now we can adjust our vertical origin
      */
@@ -513,7 +524,7 @@ XmScrollBarCallbackStruct *call_data;
 		       FILL_HORIZ_WIDTH(mw),
 		       ClipChild(mw)->core.height - height,
 		       False);
- 	
+
 	/*
 	 * Redraw the new chunk of fixed columns and row labels
 	 */
@@ -540,10 +551,7 @@ XmScrollBarCallbackStruct *call_data;
  */
 /* ARGSUSED */
 void
-xbaeScrollHorizCB(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XmScrollBarCallbackStruct *call_data;
+xbaeScrollHorizCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct *call_data)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
     Rectangle fixed, nonfixed;
@@ -560,7 +568,7 @@ XmScrollBarCallbackStruct *call_data;
     Boolean need_vert_fill = NEED_VERT_FILL(mw);
     Boolean has_attached_trailing_rows = HAS_ATTACHED_TRAILING_ROWS(mw);
     Boolean need_vert_dead_space_fill = NEED_VERT_DEAD_SPACE_FILL(mw);
-    
+
     /*printf("%d\n", HORIZ_ORIGIN(mw));*/
     /*
      * Didn't scroll
@@ -603,6 +611,27 @@ XmScrollBarCallbackStruct *call_data;
 		     TextChild(mw)->core.y);
     }
 
+    /* Cell widgets that are in the horizontal scrolling region 
+     * need to scroll along as well... Linas */
+    if (mw->matrix.cell_widgets) {
+       int roe, kol;
+
+       for (roe=0; roe < (mw->matrix.rows); roe++) { 
+          for (kol=mw->matrix.fixed_columns;
+               kol < TRAILING_HORIZ_ORIGIN(mw); kol ++) {
+             Widget w = mw->matrix.cell_widgets[roe][kol];
+             if (w) {
+                if (XtIsManaged (w)) {
+                   int x = w->core.x;
+                   int y = w->core.y;
+                   x += HORIZ_ORIGIN(mw) - call_data->value;
+                   XtMoveWidget(w, x, y);
+                }
+             }
+          }
+       }
+    }
+
     /*
      * Now we can adjust our horizontal origin
      */
@@ -668,7 +697,7 @@ XmScrollBarCallbackStruct *call_data;
 			   trailing_fixed_row_label_offset +
 			   trailing_fixed_row_height,
 			   MATRIX_HORIZ_VISIBLE_SPACE(mw), 0, False);
-	    
+
 	    if (need_vert_dead_space_fill)
 		XClearArea(XtDisplay(mw), XtWindow(mw),
 			   fixed_column_label_offset,
@@ -764,7 +793,7 @@ XmScrollBarCallbackStruct *call_data;
 		      src_x, horiz_sb_offset,
 		      width, column_label_height,
 		      dest_x, horiz_sb_offset);
-	
+
 	/*
 	 * Copy trailing filled portion of the columns if necessary
 	 */
@@ -778,7 +807,7 @@ XmScrollBarCallbackStruct *call_data;
 			  width, FILL_VERT_HEIGHT(mw),
 			  dest_x, trailing_fixed_row_label_offset +
 			  trailing_fixed_row_height);
-	    
+
 	    if (need_vert_dead_space_fill)
 		XCopyArea(XtDisplay(mw), XtWindow(mw), XtWindow(mw),
 			  mw->matrix.draw_gc,
@@ -786,7 +815,7 @@ XmScrollBarCallbackStruct *call_data;
 			  width, vert_dead_space_height,
 			  dest_x, unattached_trailing_rows_offset);
 	}
-	
+
 	/*
 	 * Clear newly scrolled chunk of fixed rows on TopClip
 	 */
@@ -856,9 +885,7 @@ XmScrollBarCallbackStruct *call_data;
  * non-fixed cells.
  */
 void
-xbaeRedrawCells(mw, expose)
-XbaeMatrixWidget mw;
-Rectangle *expose;
+xbaeRedrawCells(XbaeMatrixWidget mw, Rectangle *expose)
 {
     int startCol, endCol, startRow, endRow, i, j;
     Rectangle rect;
@@ -917,17 +944,16 @@ Rectangle *expose;
 	xbaeSetClipMask(mw, CLIP_NONE);
 }
 
+#if 0
 /*
  * Redraw the row and column labels and the cells in fixed rows/columns
  * that are overlapped by the Rectangle argument.
  */
 void
-xbaeRedrawLabelsAndFixed(mw, expose)
-XbaeMatrixWidget mw;
-Rectangle *expose;
+xbaeRedrawLabelsAndFixed(XbaeMatrixWidget mw, Rectangle *expose)
 {
     /*
-     * Set up some local variables to avoid calling too many macros :p
+     * Set up some local variables to avoid calling too many macros
      */
     int horiz_sb_offset = HORIZ_SB_OFFSET(mw);
     int vert_sb_offset = VERT_SB_OFFSET(mw);
@@ -955,14 +981,14 @@ Rectangle *expose;
     if (mw->matrix.rows && mw->matrix.fixed_rows && mw->matrix.row_labels)
     {
 	Rectangle rect;
-	
+
 	/*
 	 * Get the Rectangle enclosing the fixed row labels
 	 */
 	SETRECT(rect, vert_sb_offset, row_label_offset,
 		vert_sb_offset + row_label_width - 1,
 		fixed_row_label_offset - 1);
-	
+
 	/*
 	 * If the expose Rectangle overlaps, then some labels must be redrawn
 	 */
@@ -1594,7 +1620,7 @@ Rectangle *expose;
 		    if (redrawTrailingFixedRows && (i == skipRow) &&
 			(endRow > skipRow))
 			i = TRAILING_VERT_ORIGIN(mw);
-		    
+
 		    /*
 		     * If we need to clip the vertical fill
 		     */
@@ -1638,12 +1664,12 @@ Rectangle *expose;
 	{
 	    width = mw->core.width - row_label_width -
 		VERT_SB_SPACE(mw);
-	    
+
 	    height = mw->core.height - column_label_height -
 		HORIZ_SB_SPACE(mw) ;
 	}
-	
-	DRAW_SHADOW(XtDisplay(mw), XtWindow(mw),
+
+	DRAW_SHADOW(XtDisplay(mw), XtWindow(mw), "win",
 		    mw->manager.top_shadow_GC,
 		    mw->manager.bottom_shadow_GC,
 		    mw->manager.shadow_thickness,
@@ -1652,3 +1678,741 @@ Rectangle *expose;
 		    width, height, mw->matrix.shadow_type);
     }
 }
+#else
+/*
+ * Redraw the row and column labels and the cells in fixed rows/columns
+ * that are overlapped by the Rectangle argument.
+ *
+ * FIX ME DANNY
+ * This version is being worked on for variable height rows.
+ */
+void
+xbaeRedrawLabelsAndFixed(XbaeMatrixWidget mw, Rectangle *expose)
+{
+    /*
+     * Set up some local variables to avoid calling too many macros
+     */
+    int horiz_sb_offset = HORIZ_SB_OFFSET(mw);
+    int vert_sb_offset = VERT_SB_OFFSET(mw);
+    int column_label_height = COLUMN_LABEL_HEIGHT(mw);
+    int row_label_offset = ROW_LABEL_OFFSET(mw);
+    int row_label_width = ROW_LABEL_WIDTH(mw);
+    int fixed_row_label_offset = FIXED_ROW_LABEL_OFFSET(mw);
+    int trailing_fixed_row_label_offset = TRAILING_FIXED_ROW_LABEL_OFFSET(mw);
+    int trailing_fixed_row_height = TRAILING_FIXED_ROW_HEIGHT(mw);
+    int fixed_column_label_offset = FIXED_COLUMN_LABEL_OFFSET(mw);
+    int fixed_column_width = FIXED_COLUMN_WIDTH(mw);
+    int trailing_fixed_column_label_offset =
+	TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw);
+    int trailing_fixed_column_width = TRAILING_FIXED_COLUMN_WIDTH(mw);
+    int column_label_offset = COLUMN_LABEL_OFFSET(mw);
+    Boolean need_vert_fill = NEED_VERT_FILL(mw);
+    Boolean has_attached_trailing_rows = HAS_ATTACHED_TRAILING_ROWS(mw);
+
+    if (mw->matrix.disable_redisplay)
+	return;
+
+    /*
+     * Handle the row labels that are in fixed rows
+     */
+    if (mw->matrix.rows && mw->matrix.fixed_rows && mw->matrix.row_labels)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the fixed row labels
+	 */
+	SETRECT(rect, vert_sb_offset, row_label_offset,
+		vert_sb_offset + row_label_width - 1,
+		fixed_row_label_offset - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some labels must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int endRow, i;
+
+	    /*
+	     * Intersect the fixed-row-labels Rectangle with the expose
+	     * Rectangle along the Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    Y_INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * Redraw each label that was intersected
+	     */
+	    endRow = YtoRow(mw, intersect.y2 + mw->matrix.first_row_offset);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    for (i = YtoRow(mw, intersect.y1 + mw->matrix.first_row_offset),
+		     SANITY_CHECK_ROW(mw, i);
+		 i <= endRow; i++)
+		xbaeDrawRowLabel(mw, i, False);
+	}
+    }
+
+    /*
+     * Handle the row labels that are in trailing fixed rows
+     */
+    if (mw->matrix.rows && mw->matrix.trailing_fixed_rows &&
+	mw->matrix.row_labels)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the fixed row labels
+	 */
+	SETRECT(rect, vert_sb_offset, trailing_fixed_row_label_offset,
+		vert_sb_offset + row_label_width - 1,
+		trailing_fixed_row_label_offset +
+		trailing_fixed_row_height - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some labels must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int endRow, i;
+
+	    /*
+	     * Intersect the fixed-row-labels Rectangle with the expose
+	     * Rectangle along the Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    Y_INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * Redraw each label that was intersected
+	     */
+	    endRow = YtoRow(mw, intersect.y2) + TRAILING_VERT_ORIGIN(mw);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    for (i = YtoRow(mw, intersect.y1) + TRAILING_VERT_ORIGIN(mw),
+		 SANITY_CHECK_ROW(mw, i); i <= endRow; i++)
+		xbaeDrawRowLabel(mw, i, False);
+	}
+    }
+
+    /*
+     * Handle row labels that aren't in fixed rows
+     */
+    if (mw->matrix.row_labels && mw->matrix.rows)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the non-fixed row labels
+	 */
+	SETRECT(rect, vert_sb_offset, fixed_row_label_offset,
+		vert_sb_offset + row_label_width - 1,
+		fixed_row_label_offset + VISIBLE_HEIGHT(mw) - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some labels must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int endRow, i;
+
+	    /*
+	     * Intersect the fixed-row-labels Rectangle with the expose
+	     * Rectangle along the Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    Y_INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * Translate 'intersect' to take into account any fixed rows.
+	     * This gets it back into the coord system expected by YtoRow().
+	     */
+	    intersect.y1 += FIXED_ROW_HEIGHT(mw);
+	    intersect.y2 += FIXED_ROW_HEIGHT(mw);
+
+	    /*
+	     * Redraw each label that was intersected
+	     */
+	    endRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    for (i = YtoRow(mw, intersect.y1) + VERT_ORIGIN(mw),
+		 SANITY_CHECK_ROW(mw, i); i <= endRow;	 i++)
+		xbaeDrawRowLabel(mw, i, False);
+	}
+    }
+
+    /*
+     * Handle the column labels that are in fixed columns
+     */
+    if (mw->matrix.columns && mw->matrix.fixed_columns &&
+	mw->matrix.column_labels)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the portion of the column labels
+	 * that are in fixed columns
+	 */
+	SETRECT(rect,
+		column_label_offset, horiz_sb_offset,
+		fixed_column_label_offset - 1,
+		horiz_sb_offset + column_label_height - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some labels must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int endCol, i;
+
+	    /*
+	     * Intersect the fixed-column-labels Rectangle with the expose
+	     * Rectangle along the X axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    X_INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * Redraw each label that was intersected
+	     */
+	    endCol = xbaeXtoCol(mw, intersect.x2);
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    for (i = xbaeXtoCol(mw, intersect.x1), SANITY_CHECK_COLUMN(mw, i);
+		 i <= endCol; i++)
+		xbaeDrawColumnLabel(mw, i, False);
+	}
+    }
+
+    /*
+     * Handle the column labels that are in trailing fixed columns
+     */
+    if (mw->matrix.columns && mw->matrix.trailing_fixed_columns &&
+	mw->matrix.column_labels)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the portion of the column labels
+	 * that are in fixed columns
+	 */
+	SETRECT(rect,
+		trailing_fixed_column_label_offset, horiz_sb_offset,
+		trailing_fixed_column_label_offset +
+		trailing_fixed_column_width,
+		horiz_sb_offset + column_label_height);
+
+	/*
+	 * If the expose Rectangle overlaps, then some labels must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int endCol, i;
+
+	    /*
+	     * Intersect the fixed-column-labels Rectangle with the expose
+	     * Rectangle along the X axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    X_INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * Redraw each label that was intersected
+	     */
+	    endCol = xbaeXtoTrailingCol(mw, intersect.x2);
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    for (i = xbaeXtoTrailingCol(mw, intersect.x1),
+		 SANITY_CHECK_COLUMN(mw, i); i <= endCol; i++)
+		xbaeDrawColumnLabel(mw, i, False);
+	}
+    }
+
+    /*
+     * Handle column labels that aren't in fixed columns
+     */
+    if (mw->matrix.column_labels && mw->matrix.columns)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the non-fixed column labels
+	 */
+	SETRECT(rect,
+		fixed_column_label_offset, horiz_sb_offset,
+		fixed_column_label_offset + VISIBLE_WIDTH(mw) - 1,
+		horiz_sb_offset + column_label_height - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some labels must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int endCol, i;
+
+	    /*
+	     * Intersect the non-fixed-column-labels Rectangle with the expose
+	     * Rectangle along the X axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    X_INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * Translate 'intersect' to take into account any fixed columns.
+	     * This gets it back into the coord system expected by XtoCol().
+	     */
+	    intersect.x1 += fixed_column_width;
+	    intersect.x2 += fixed_column_width;
+
+	    /*
+	     * Redraw each label that was intersected
+	     */
+	    endCol = xbaeXtoCol(mw, intersect.x2 + HORIZ_ORIGIN(mw));
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    for (i = xbaeXtoCol(mw, intersect.x1 + HORIZ_ORIGIN(mw)),
+		 SANITY_CHECK_COLUMN(mw, i); i <= endCol; i++)
+		xbaeDrawColumnLabel(mw, i, False);
+	}
+    }
+
+    /*
+     * Handle cells in fixed rows except those also in fixed columns
+     */
+    if (mw->matrix.rows && mw->matrix.columns && mw->matrix.fixed_rows)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the cells in fixed rows
+	 */
+	SETRECT(rect,
+		fixed_column_label_offset, row_label_offset,
+		fixed_column_label_offset + VISIBLE_WIDTH(mw) - 1,
+		fixed_row_label_offset - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some cells must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int startCol, endCol, startRow, endRow, i, j;
+
+	    /*
+	     * Intersect the fixed-cells Rectangle with the expose
+	     * Rectangle along the X and Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    INTERSECT(rect, *expose, intersect);
+
+	    intersect.x1 += fixed_column_width;
+	    intersect.x2 += fixed_column_width;
+
+	    /*
+	     * Get starting and ending rows/columns. Always take into
+	     * account the scrolling origins for the columns; for rows
+	     * only if we are fixed in that dimension.
+	     */
+	    startCol = xbaeXtoCol(mw, intersect.x1 + HORIZ_ORIGIN(mw));
+	    endCol = xbaeXtoCol(mw, intersect.x2 + HORIZ_ORIGIN(mw));
+
+	    startRow = YtoRow(mw, intersect.y1) +
+		(mw->matrix.fixed_rows
+		 ? 0
+		 : VERT_ORIGIN(mw));
+	    endRow = YtoRow(mw, intersect.y2) +
+		(mw->matrix.fixed_rows
+		 ? 0
+		 : VERT_ORIGIN(mw));
+
+	    /*
+	     * Redraw each cell that was intersected
+	     */
+	    SANITY_CHECK_ROW(mw, startRow);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    SANITY_CHECK_COLUMN(mw, startCol);
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    for (i = startRow; i <= endRow; i++)
+		for (j = startCol; j <= endCol; j++)
+		    xbaeDrawCell(mw, i, j);
+	}
+    }
+
+    /*
+     * Handle cells in trailing fixed rows except those also in fixed columns
+     */
+    if (mw->matrix.rows && mw->matrix.columns &&
+	mw->matrix.trailing_fixed_rows)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the cells in trailing fixed rows
+	 */
+	SETRECT(rect, fixed_column_label_offset,
+		trailing_fixed_row_label_offset,
+		fixed_column_label_offset + VISIBLE_WIDTH(mw) - 1,
+		trailing_fixed_row_label_offset +
+		trailing_fixed_row_height - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some cells must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int startCol, endCol, startRow, endRow, i, j;
+
+	    /*
+	     * Intersect the fixed-cells Rectangle with the expose
+	     * Rectangle along the X and Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    INTERSECT(rect, *expose, intersect);
+
+	    intersect.x1 += fixed_column_width;
+	    intersect.x2 += fixed_column_width;
+
+	    /*
+	     * Get starting and ending rows/columns. Always take into
+	     * account the scrolling origins for the columns and never
+	     * for the rows.
+	     */
+	    startCol = xbaeXtoCol(mw, intersect.x1 + HORIZ_ORIGIN(mw));
+	    endCol = xbaeXtoCol(mw, intersect.x2 + HORIZ_ORIGIN(mw));
+
+	    startRow = YtoRow(mw, intersect.y1) + TRAILING_VERT_ORIGIN(mw);
+	    endRow = YtoRow(mw, intersect.y2) + TRAILING_VERT_ORIGIN(mw);
+
+	    /*
+	     * Redraw each cell that was intersected
+	     */
+	    SANITY_CHECK_ROW(mw, startRow);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    SANITY_CHECK_COLUMN(mw, startCol);
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    xbaeSetClipMask(mw, CLIP_TRAILING_FIXED_ROWS);
+	    for (i = startRow; i <= endRow; i++)
+		for (j = startCol; j <= endCol; j++)
+		    xbaeDrawCell(mw, i, j);
+	    xbaeSetClipMask(mw, CLIP_NONE);
+	}
+    }
+
+    /*
+     * Handle cells in fixed columns
+     */
+    if (mw->matrix.rows && mw->matrix.columns && mw->matrix.fixed_columns)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the cells in fixed columns
+	 */
+	SETRECT(rect,
+		column_label_offset, row_label_offset,
+		fixed_column_label_offset - 1,
+		trailing_fixed_row_label_offset +
+		trailing_fixed_row_height - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some cells must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int startCol, endCol, startRow, endRow, skipRow = -1, i, j;
+	    Boolean redrawFixedRows, redrawTrailingFixedRows;
+	    unsigned int clip_reason = CLIP_NONE;
+
+	    /*
+	     * Intersect the fixed-cells Rectangle with the expose
+	     * Rectangle along the X and Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * If we have any fixed rows, we might need to redraw the cells
+	     * located in the intersection of the fixed rows and columns.
+	     * These cells may force use to be different than our current
+	     * VERT_ORIGIN.
+	     */
+	    redrawFixedRows = redrawTrailingFixedRows = False;
+	    if (mw->matrix.fixed_rows)
+	    {
+		clip_reason = CLIP_FIXED_COLUMNS;
+
+		SETRECT(rect,
+			column_label_offset, row_label_offset,
+			fixed_column_label_offset - 1,
+			fixed_row_label_offset - 1);
+
+		if (OVERLAP(*expose, rect))
+		    redrawFixedRows = True;
+	    }
+
+	    if (mw->matrix.trailing_fixed_rows)
+	    {
+		clip_reason = CLIP_FIXED_COLUMNS;
+
+		SETRECT(rect,
+			column_label_offset,
+			trailing_fixed_row_label_offset,
+			fixed_column_label_offset - 1,
+			trailing_fixed_row_label_offset +
+			trailing_fixed_row_height - 1);
+
+		if (OVERLAP(*expose, rect))
+		    redrawTrailingFixedRows = True;
+	    }
+
+	    if (CLIP_NONE != clip_reason)
+		xbaeSetClipMask(mw, clip_reason);
+
+	    /*
+	     * Get starting and ending rows/columns. Never take into
+	     * account the scrolling origins for the rows; for columns
+	     * only if we are fixed in that dimension.
+	     */
+	    startCol = xbaeXtoCol(mw, intersect.x1 +
+			      (mw->matrix.fixed_columns
+			       ? 0
+			       : HORIZ_ORIGIN(mw)));
+	    endCol = xbaeXtoCol(mw, intersect.x2 +
+			    (mw->matrix.fixed_columns
+			     ? 0
+			     : HORIZ_ORIGIN(mw)));
+
+	    startRow = redrawFixedRows ? 0 : YtoRow(mw, intersect.y1) +
+		VERT_ORIGIN(mw);
+	    if (redrawTrailingFixedRows)
+	    {
+		skipRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw) -
+		    mw->matrix.trailing_fixed_rows + 1;
+		endRow = mw->matrix.rows - 1;
+	    }
+	    else
+		endRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
+
+	    /*
+	     * Redraw each cell that was intersected
+	     */
+	    SANITY_CHECK_ROW(mw, startRow);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    SANITY_CHECK_COLUMN(mw, startCol);
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    for (i = startRow; i <= endRow; i++)
+		for (j = startCol; j <= endCol; j++)
+		{
+		    /*
+		     * If we had to redraw cells located in both fixed rows
+		     * and columns, when we are done redrawing those cells,
+		     * we need to skip to the correct non-fixed row to draw,
+		     * or alternatively, jump to the trailing fixed row
+		     * to draw.
+		     */
+		    if (redrawFixedRows && i == mw->matrix.fixed_rows)
+			i += VERT_ORIGIN(mw);
+
+		    if (redrawTrailingFixedRows && i == skipRow &&
+			endRow > skipRow && i < TRAILING_VERT_ORIGIN(mw))
+			i = TRAILING_VERT_ORIGIN(mw);
+
+		    /*
+		     * If we need to clip the vertical fill
+		     */
+		    if (!(clip_reason & CLIP_TRAILING_FIXED_ROWS) &&
+			IN_GRID_COLUMN_MODE(mw) &&
+			((mw->matrix.rows - 1) == i) && 
+			(need_vert_fill && (! has_attached_trailing_rows)))
+		    {
+			clip_reason |= CLIP_FIXED_COLUMNS |
+			    CLIP_TRAILING_FIXED_ROWS;
+			xbaeSetClipMask(mw, clip_reason);
+		    }
+
+		    xbaeDrawCell(mw, i, j);
+		}
+
+	    if (CLIP_NONE != clip_reason)
+		xbaeSetClipMask(mw, CLIP_NONE);
+	}
+    }
+
+    /*
+     * Handle cells in trailing fixed columns
+     */
+    if (mw->matrix.rows && mw->matrix.columns &&
+	mw->matrix.trailing_fixed_columns)
+    {
+	Rectangle rect;
+
+	/*
+	 * Get the Rectangle enclosing the cells in trailing fixed columns
+	 */
+	SETRECT(rect,
+		trailing_fixed_column_label_offset, row_label_offset,
+		trailing_fixed_column_label_offset +
+		trailing_fixed_column_width - 1,
+		trailing_fixed_row_label_offset +
+		trailing_fixed_row_height - 1);
+
+	/*
+	 * If the expose Rectangle overlaps, then some cells must be redrawn
+	 */
+	if (OVERLAP(*expose, rect))
+	{
+	    Rectangle intersect;
+	    int startCol, endCol, startRow, endRow, skipRow = -1, i, j;
+	    Boolean redrawFixedRows, redrawTrailingFixedRows;
+	    unsigned int clip_reason = CLIP_NONE;
+
+	    /*
+	     * Intersect the fixed-cells Rectangle with the expose
+	     * Rectangle along the X and Y axis.  The resulting Rectangle will
+	     * be in 'rect's coordinate system.
+	     */
+	    INTERSECT(rect, *expose, intersect);
+
+	    /*
+	     * If we have any fixed rows, we might need to redraw the cells
+	     * located in the intersection of the fixed rows and columns.
+	     * These cells may force us to be different than our current
+	     * VERT_ORIGIN.
+	     */
+	    redrawFixedRows = redrawTrailingFixedRows = False;
+	    if (mw->matrix.fixed_rows)
+	    {
+		clip_reason = CLIP_FIXED_COLUMNS;
+
+		SETRECT(rect, trailing_fixed_column_label_offset,
+			row_label_offset,
+			trailing_fixed_column_label_offset +
+			trailing_fixed_column_width - 1,
+			fixed_row_label_offset - 1);
+
+		if (OVERLAP(*expose, rect))
+		    redrawFixedRows = True;
+	    }
+
+	    if (mw->matrix.trailing_fixed_rows)
+	    {
+		clip_reason = CLIP_FIXED_COLUMNS;
+
+		SETRECT(rect, trailing_fixed_column_label_offset,
+			trailing_fixed_row_label_offset,
+			trailing_fixed_column_label_offset +
+			trailing_fixed_column_width - 1,
+			trailing_fixed_row_label_offset +
+			trailing_fixed_row_height - 1);
+
+		if (OVERLAP(*expose, rect))
+		    redrawTrailingFixedRows = True;
+	    }
+
+	    if (CLIP_NONE != clip_reason)
+		xbaeSetClipMask(mw, clip_reason);
+
+	    /*
+	     * Get starting and ending rows/columns. Never take into
+	     * account the scrolling origins for the rows; for columns
+	     * only if we are fixed in that dimension.
+	     */
+	    startCol = xbaeXtoTrailingCol(mw, intersect.x1);
+	    endCol = xbaeXtoTrailingCol(mw, intersect.x2);
+
+	    startRow = redrawFixedRows ? 0 : YtoRow(mw, intersect.y1) +
+		VERT_ORIGIN(mw);
+
+	    if (redrawTrailingFixedRows)
+	    {
+		skipRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw) -
+		    mw->matrix.trailing_fixed_rows + 1;
+		endRow = mw->matrix.rows - 1;
+	    }
+	    else
+		endRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
+
+	    /*
+	     * Redraw each cell that was intersected
+	     */
+	    SANITY_CHECK_ROW(mw, startRow);
+	    SANITY_CHECK_ROW(mw, endRow);
+	    SANITY_CHECK_COLUMN(mw, startCol);
+	    SANITY_CHECK_COLUMN(mw, endCol);
+	    for (i = startRow; i <= endRow; i++)
+		for (j = startCol; j <= endCol; j++)
+		{
+		    /*
+		     * If we had to redraw cells located in both fixed rows
+		     * and columns, when we are done redrawing those cells,
+		     * we need to skip to the correct non-fixed row to draw
+		     */		    
+		    if (redrawFixedRows && (i == mw->matrix.fixed_rows))
+			i += VERT_ORIGIN(mw);
+
+		    if (redrawTrailingFixedRows && (i == skipRow) &&
+			(endRow > skipRow))
+			i = TRAILING_VERT_ORIGIN(mw);
+
+		    /*
+		     * If we need to clip the vertical fill
+		     */
+		    if (!(clip_reason & CLIP_TRAILING_FIXED_ROWS) &&
+			IN_GRID_COLUMN_MODE(mw) &&
+			((mw->matrix.rows - 1) == i) && 
+			(need_vert_fill && (! has_attached_trailing_rows)))
+		    {
+			clip_reason |= CLIP_TRAILING_FIXED_COLUMNS |
+			    CLIP_TRAILING_FIXED_ROWS;
+			xbaeSetClipMask(mw, clip_reason);
+		    }
+
+		    xbaeDrawCell(mw, i, j);
+		}
+
+	    if (CLIP_NONE != clip_reason)
+		xbaeSetClipMask(mw, CLIP_NONE);
+	}
+    }
+
+    /*
+     * Draw a shadow just inside row/column labels and around outer edge
+     * of clip widget.	We can't use height of clip widget because it is
+     * truncated to nearest row.  We use cell_visible_height instead.
+     */
+    if (mw->manager.shadow_thickness)
+    {
+	Dimension width, height;
+
+	if (! mw->matrix.fill)
+	{
+	    width = ClipChild(mw)->core.width + fixed_column_width +
+		trailing_fixed_column_width +
+		2 * mw->manager.shadow_thickness;
+	    height = mw->matrix.cell_visible_height + FIXED_ROW_HEIGHT(mw) +
+		trailing_fixed_row_height +
+		2 * mw->manager.shadow_thickness;
+	}
+	else
+	{
+	    width = mw->core.width - row_label_width -
+		VERT_SB_SPACE(mw);
+
+	    height = mw->core.height - column_label_height -
+		HORIZ_SB_SPACE(mw) ;
+	}
+
+	DRAW_SHADOW(XtDisplay(mw), XtWindow(mw), "win",
+		    mw->manager.top_shadow_GC,
+		    mw->manager.bottom_shadow_GC,
+		    mw->manager.shadow_thickness,
+		    row_label_width + vert_sb_offset,
+		    column_label_height + horiz_sb_offset,
+		    width, height, mw->matrix.shadow_type);
+    }
+}
+#endif
