@@ -20,7 +20,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Shadow.c,v 1.9 1999/07/24 02:03:48 lister Exp $
+ * $Id: Shadow.c,v 1.3 2000/03/28 21:44:11 danny Exp $
  */
 
 
@@ -30,7 +30,7 @@
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <Xm/Xm.h>
@@ -43,37 +43,21 @@
 #include <Xbae/Draw.h>
 #include <Xbae/Utils.h>
 
-static void DrawRowShadow P((XbaeMatrixWidget, Window, int, int, int,
-			     int, int, int, GC, GC));
+static void DrawRowShadow(XbaeMatrixWidget, Window, int, int, int,
+			     int, int, int, GC, GC);
 
-static void DrawColumnShadow P((XbaeMatrixWidget, Window, int, int, int,
-				int, int, int, GC, GC));
+static void DrawColumnShadow(XbaeMatrixWidget, Window, int, int, int,
+				int, int, int, GC, GC);
 
-static void DrawRowHighlight P((XbaeMatrixWidget, Window, GC, int, int,
-				 int, int, int, int, int));
-static void DrawColumnHighlight P((XbaeMatrixWidget, Window, GC, int,
-				    int, int, int, int, int, int));
+static void DrawRowHighlight(XbaeMatrixWidget, Window, GC, int, int,
+				 int, int, int, int, int);
+static void DrawColumnHighlight(XbaeMatrixWidget, Window, GC, int,
+				    int, int, int, int, int, int);
 
 void
-#if NeedFunctionPrototypes
 xbaeDrawCellShadow(XbaeMatrixWidget mw, Window win, int row, int column, int x,
 		   int y, int width, int height, Boolean label,
 		   Boolean clipped, Boolean pressed)
-#else
-xbaeDrawCellShadow(mw, win, row, column, x, y, width, height, label, clipped,
-		   pressed)
-XbaeMatrixWidget mw;
-Window win;
-int row;
-int column;
-int x;
-int y;
-int width;
-int height;
-Boolean label;
-Boolean clipped;
-Boolean pressed;
-#endif
 {
     unsigned char grid_type;
     unsigned char shadow;
@@ -103,7 +87,7 @@ Boolean pressed;
 	switch (grid_type)
 	{
 	    case XmGRID_CELL_SHADOW:
-		DRAW_SHADOW(XtDisplay(mw), win,
+		DRAW_SHADOW(XtDisplay(mw), win, "win",
 			    mw->matrix.cell_top_shadow_clip_gc,
 			    mw->matrix.cell_bottom_shadow_clip_gc,
 			    mw->matrix.cell_shadow_thickness,
@@ -112,14 +96,14 @@ Boolean pressed;
 
 	    /* Deprecated types. To be removed in next version. */
 	    case XmGRID_SHADOW_OUT:
-		DRAW_SHADOW(XtDisplay(mw), win,
+		DRAW_SHADOW(XtDisplay(mw), win, "win",
 			    mw->matrix.cell_bottom_shadow_clip_gc,
 			    mw->matrix.cell_top_shadow_clip_gc,
 			    mw->matrix.cell_shadow_thickness,
 			    x, y, width, height, shadow);
 		break;
 	    case XmGRID_SHADOW_IN:
-		DRAW_SHADOW(XtDisplay(mw), win,
+		DRAW_SHADOW(XtDisplay(mw), win, "win",
 			    mw->matrix.cell_top_shadow_clip_gc,
 			    mw->matrix.cell_bottom_shadow_clip_gc,
 			    mw->matrix.cell_shadow_thickness,
@@ -152,14 +136,14 @@ Boolean pressed;
 			     mw->matrix.grid_line_gc, mw->matrix.grid_line_gc);
 	    break;
 	case XmGRID_CELL_LINE:
-	    DRAW_SHADOW(XtDisplay(mw), win,
+	    DRAW_SHADOW(XtDisplay(mw), win, "win",
 			mw->matrix.grid_line_gc,
 			mw->matrix.grid_line_gc,
 			mw->matrix.cell_shadow_thickness,
 			x, y, width, height, shadow);
 	    break;
 	case XmGRID_CELL_SHADOW:
-	    DRAW_SHADOW(XtDisplay(mw), win,
+	    DRAW_SHADOW(XtDisplay(mw), win, "win",
 			mw->manager.top_shadow_GC,
 			mw->manager.bottom_shadow_GC,
 			mw->matrix.cell_shadow_thickness,
@@ -168,21 +152,21 @@ Boolean pressed;
 
 	/* Deprecated types. To be removed in next version. */
 	case XmGRID_LINE:
-	    DRAW_SHADOW(XtDisplay(mw), win,
+	    DRAW_SHADOW(XtDisplay(mw), win, "win",
 			mw->matrix.grid_line_gc,
 			mw->matrix.grid_line_gc,
 			mw->matrix.cell_shadow_thickness,
 			x, y, width, height, shadow);
 	    break;
 	case XmGRID_SHADOW_OUT:
-	    DRAW_SHADOW(XtDisplay(mw), win,
+	    DRAW_SHADOW(XtDisplay(mw), win, "win",
 			mw->manager.bottom_shadow_GC,
 			mw->manager.top_shadow_GC,
 			mw->matrix.cell_shadow_thickness,
 			x, y, width, height, shadow);
 	    break;
 	case XmGRID_SHADOW_IN:
-	    DRAW_SHADOW(XtDisplay(mw), win,
+	    DRAW_SHADOW(XtDisplay(mw), win, "win",
 			mw->manager.top_shadow_GC,
 			mw->manager.bottom_shadow_GC,
 			mw->matrix.cell_shadow_thickness,
@@ -194,17 +178,16 @@ Boolean pressed;
 
 #if XmVersion >= 1002
 void
-xbaeDrawCellHighlight(mw, win, gc, row, column, x, y, width, height, reason)
-XbaeMatrixWidget mw;
-Window win;
-GC gc;
-int row;
-int column;
-int x;
-int y;
-int width;
-int height;
-int reason;
+xbaeDrawCellHighlight(XbaeMatrixWidget mw,
+	Window win,
+	GC gc,
+	int row,
+	int column,
+	int x,
+	int y,
+	int width,
+	int height,
+	int reason)
 {    
     int thick;
 
@@ -241,18 +224,17 @@ int reason;
 }
 
 static void
-DrawRowHighlight(mw, win, gc, row, column, x, y, width, height, reason)
-XbaeMatrixWidget mw;
-Window win;
-GC gc;
-int row;
-int column;
-int x;
-int y;
-int width;
-int height;
-int reason;
-{    
+DrawRowHighlight(XbaeMatrixWidget mw,
+	Window win,
+	GC gc,
+	int row,
+	int column,
+	int x,
+	int y,
+	int width,
+	int height,
+	int reason)
+{
     XRectangle rect[1];
     
     rect[0].x = 0;
@@ -314,18 +296,17 @@ int reason;
 }
 
 static void
-DrawColumnHighlight(mw, win, gc, row, column, x, y, width, height, reason)
-XbaeMatrixWidget mw;
-Window win;
-GC gc;
-int row;
-int column;
-int x;
-int y;
-int width;
-int height;
-int reason;
-{    
+DrawColumnHighlight(XbaeMatrixWidget mw,
+	Window win,
+	GC gc,
+	int row,
+	int column,
+	int x,
+	int y,
+	int width,
+	int height,
+	int reason)
+{
     XRectangle rect[1];
     int vert_dead_space_height = VERT_DEAD_SPACE_HEIGHT(mw);
     int clip_vert_visible_space = CLIP_VERT_VISIBLE_SPACE(mw);
@@ -419,17 +400,16 @@ int reason;
 #endif
 
 static void
-DrawRowShadow(mw, win, row, column, x, y, width, height, topGC, bottomGC)
-XbaeMatrixWidget mw;
-Window win;
-int row;
-int column;
-int x;
-int y;
-int width;
-int height;
-GC  topGC;
-GC  bottomGC;
+DrawRowShadow(XbaeMatrixWidget mw,
+	Window win,
+	int row,
+	int column,
+	int x,
+	int y,
+	int width,
+	int height,
+	GC  topGC,
+	GC  bottomGC)
 {
     XRectangle rect[1];
     unsigned char shadow = mw->matrix.row_shadow_types ?
@@ -463,7 +443,7 @@ GC  bottomGC;
 	if (column != 0)
 	    x -= mw->matrix.cell_shadow_thickness;
 
-	DRAW_SHADOW(XtDisplay(mw), win,
+	DRAW_SHADOW(XtDisplay(mw), win, "win",
 		    topGC, bottomGC,
 		    mw->matrix.cell_shadow_thickness,
 		    x, y, width, height, shadow);
@@ -479,7 +459,7 @@ GC  bottomGC;
 	else
 	    width += mw->matrix.cell_shadow_thickness;
 
-	DRAW_SHADOW(XtDisplay(mw), win,
+	DRAW_SHADOW(XtDisplay(mw), win, "win",
 		    topGC, bottomGC,
 		    mw->matrix.cell_shadow_thickness,
 		    x - mw->matrix.cell_shadow_thickness, y, width,
@@ -549,7 +529,7 @@ GC  bottomGC;
 	    /*
 	     * Draw the remaining shadow directly onto the matrix window
 	     */
-	    DRAW_SHADOW(XtDisplay(mw), XtWindow(mw),
+	    DRAW_SHADOW(XtDisplay(mw), XtWindow(mw), "win",
 			topGC, bottomGC,
 			mw->matrix.cell_shadow_thickness,
 			ax - mw->matrix.cell_shadow_thickness,
@@ -568,17 +548,16 @@ GC  bottomGC;
 }
 
 static void
-DrawColumnShadow(mw, win, row, column, x, y, width, height, topGC, bottomGC)
-XbaeMatrixWidget mw;
-Window win;
-int row;
-int column;
-int x;
-int y;
-int width;
-int height;
-GC  topGC;
-GC  bottomGC;
+DrawColumnShadow(XbaeMatrixWidget mw,
+	Window win,
+	int row,
+	int column,
+	int x,
+	int y,
+	int width,
+	int height,
+	GC  topGC,
+	GC  bottomGC)
 {
     XRectangle rect[1];
     unsigned char shadow = mw->matrix.column_shadow_types ?
@@ -625,7 +604,7 @@ GC  bottomGC;
 	if (row != 0)
 	    y -= mw->matrix.cell_shadow_thickness;
 
-	DRAW_SHADOW(XtDisplay(mw), win,
+	DRAW_SHADOW(XtDisplay(mw), win, "win",
 		    topGC, bottomGC,
 		    mw->matrix.cell_shadow_thickness,
 		    x, y, width, height, shadow);
@@ -641,7 +620,7 @@ GC  bottomGC;
 	else
 	    height += mw->matrix.cell_shadow_thickness;
 
-	DRAW_SHADOW(XtDisplay(mw), win,
+	DRAW_SHADOW(XtDisplay(mw), win, "win",
 		    topGC, bottomGC,
 		    mw->matrix.cell_shadow_thickness,
 		    x, y - mw->matrix.cell_shadow_thickness,
@@ -741,7 +720,7 @@ GC  bottomGC;
 	    /*
 	     * Draw the remaining shadow directly onto the matrix window
 	     */
-	    DRAW_SHADOW(XtDisplay(mw), XtWindow(mw),
+	    DRAW_SHADOW(XtDisplay(mw), XtWindow(mw), "win",
 			topGC, bottomGC,
 			mw->matrix.cell_shadow_thickness,
 			ax, ay - mw->matrix.cell_shadow_thickness,
