@@ -22,7 +22,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Utils.c,v 1.16 1999/07/24 02:03:49 lister Exp $
+ * $Id: Utils.c,v 1.6 2000/03/21 23:02:09 danny Exp $
  */
 
 /*
@@ -30,7 +30,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <XbaeConfig.h>
 #endif
 
 #include <Xm/Xm.h>
@@ -43,9 +43,7 @@
  * Return the top and bottom-most visible non-fixed row
  */
 void
-xbaeGetVisibleRows(mw, top_row, bottom_row)
-XbaeMatrixWidget mw;
-int *top_row, *bottom_row;
+xbaeGetVisibleRows(XbaeMatrixWidget mw, int *top_row, int *bottom_row)
 {
     *top_row = VERT_ORIGIN(mw) + mw->matrix.fixed_rows;
     *bottom_row = *top_row + (VISIBLE_HEIGHT(mw) - 1) / ROW_HEIGHT(mw);
@@ -56,9 +54,7 @@ int *top_row, *bottom_row;
  * Return the left and right-most visible non-fixed column
  */
 void
-xbaeGetVisibleColumns(mw, left_column, right_column)
-XbaeMatrixWidget mw;
-int *left_column, *right_column;
+xbaeGetVisibleColumns(XbaeMatrixWidget mw, int *left_column, int *right_column)
 {
     *left_column = xbaeXtoCol(mw, FIXED_COLUMN_WIDTH(mw) + HORIZ_ORIGIN(mw));
     *right_column = xbaeXtoCol(mw, FIXED_COLUMN_WIDTH(mw) + HORIZ_ORIGIN(mw) +
@@ -70,9 +66,9 @@ int *left_column, *right_column;
  * the visible non-fixed cells
  */
 void
-xbaeGetVisibleCells(mw, top_row, bottom_row, left_column, right_column)
-XbaeMatrixWidget mw;
-int *top_row, *bottom_row, *left_column, *right_column;
+xbaeGetVisibleCells(XbaeMatrixWidget mw,
+	int *top_row, int *bottom_row,
+	int *left_column, int *right_column)
 {
     xbaeGetVisibleRows(mw, top_row, bottom_row);
     xbaeGetVisibleColumns(mw, left_column, right_column);
@@ -85,8 +81,7 @@ int *top_row, *bottom_row, *left_column, *right_column;
  * If we can't make leftColumn the left column, make it as close as possible.
  */
 void
-xbaeAdjustLeftColumn(mw)
-XbaeMatrixWidget mw;
+xbaeAdjustLeftColumn(XbaeMatrixWidget mw)
 {
     int y;
     int i;
@@ -138,8 +133,7 @@ XbaeMatrixWidget mw;
  * If we can't make topRow the top row, make it as close as possible.
  */
 void
-xbaeAdjustTopRow(mw)
-XbaeMatrixWidget mw;
+xbaeAdjustTopRow(XbaeMatrixWidget mw)
 {
     int rows_visible = VISIBLE_HEIGHT(mw) / ROW_HEIGHT(mw);
 
@@ -169,9 +163,7 @@ XbaeMatrixWidget mw;
  * Does not check if the cell is actually visible before clearing it.
  */
 void
-xbaeClearCell(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeClearCell(XbaeMatrixWidget mw, int row, int column)
 {
     int x, y;
     Boolean fixed = IS_FIXED(mw, row, column);
@@ -200,9 +192,7 @@ int row, column;
  * Return True if a row is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsRowVisible(mw, row)
-XbaeMatrixWidget mw;
-int row;
+xbaeIsRowVisible(XbaeMatrixWidget mw, int row)
 {
     /*
      * If we are not in a fixed row or trailing fixed row,
@@ -238,9 +228,7 @@ int row;
  * Return True if a column is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsColumnVisible(mw, column)
-XbaeMatrixWidget mw;
-int column;
+xbaeIsColumnVisible(XbaeMatrixWidget mw, int column)
 {
     /*
      * If we are not in a fixed column, see if we are on the screen
@@ -273,9 +261,7 @@ int column;
  * Return True if a cell is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsCellVisible(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeIsCellVisible(XbaeMatrixWidget mw, int row, int column)
 {
     return xbaeIsRowVisible(mw, row) && xbaeIsColumnVisible(mw, column);
 }
@@ -284,9 +270,7 @@ int row, column;
  * Scroll a row so it is visible on the screen.
  */
 void
-xbaeMakeRowVisible(mw, row)
-XbaeMatrixWidget mw;
-int row;
+xbaeMakeRowVisible(XbaeMatrixWidget mw, int row)
 {
     int rows_visible;
     int value, slider_size, increment, page_increment, vert_value;
@@ -335,9 +319,7 @@ int row;
  * Scroll a column so it is visible on the screen.
  */
 void
-xbaeMakeColumnVisible(mw, column)
-XbaeMatrixWidget mw;
-int column;
+xbaeMakeColumnVisible(XbaeMatrixWidget mw, int column)
 {
     int value, slider_size, increment, page_increment, x, horiz_value;
 
@@ -363,13 +345,13 @@ int column;
     else if (x + COLUMN_WIDTH(mw, column) >
 	     VISIBLE_WIDTH(mw) + HORIZ_ORIGIN(mw))
     {
-	int off = (x + COLUMN_WIDTH(mw, column)) - (VISIBLE_WIDTH(mw) +
+	int offs = (x + COLUMN_WIDTH(mw, column)) - (VISIBLE_WIDTH(mw) +
 						    HORIZ_ORIGIN(mw));
 
-	if (x - off < HORIZ_ORIGIN(mw))
+	if (x - offs < HORIZ_ORIGIN(mw))
 	    horiz_value = x;
 	else
-	    horiz_value = HORIZ_ORIGIN(mw) + off;
+	    horiz_value = HORIZ_ORIGIN(mw) + offs;
     }
     else
 	horiz_value = HORIZ_ORIGIN(mw);
@@ -391,9 +373,7 @@ int column;
  * Scrolls a fixed or non-fixed cell so it is visible on the screen.
  */
 void
-xbaeMakeCellVisible(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeMakeCellVisible(XbaeMatrixWidget mw, int row, int column)
 {
     if (!xbaeIsRowVisible(mw, row))
 	xbaeMakeRowVisible(mw, row);
@@ -406,9 +386,7 @@ int row, column;
  * drawing non-fixed column labels and fixed rows.
  */
 void
-xbaeSetClipMask(mw, clip_reason)
-XbaeMatrixWidget mw;
-unsigned int clip_reason;
+xbaeSetClipMask(XbaeMatrixWidget mw, unsigned int clip_reason)
 {
     XRectangle r[2];
     int n = 1;
@@ -488,13 +466,11 @@ unsigned int clip_reason;
 		       0, 0, r, n, Unsorted);
 }
 
-
 /*
  * Get the total pixel width of the non-fixed cell area
  */
 void
-xbaeGetCellTotalWidth(mw)
-XbaeMatrixWidget mw;
+xbaeGetCellTotalWidth(XbaeMatrixWidget mw)
 {
     int i, columns;
 
@@ -512,8 +488,7 @@ XbaeMatrixWidget mw;
  * Cache the pixel position of each column
  */
 void
-xbaeGetColumnPositions(mw)
-XbaeMatrixWidget mw;
+xbaeGetColumnPositions(XbaeMatrixWidget mw)
 {
     int i, x;
 
@@ -524,20 +499,24 @@ XbaeMatrixWidget mw;
 }
 
 void
-#if NeedFunctionPrototypes
+xbaeGetRowPositions(XbaeMatrixWidget mw)
+{
+    int i, y;
+
+    for (i = 0, y = 0;
+	 i < mw->matrix.rows;
+	 y += SOME_ROW_HEIGHT(mw, i), i++)
+	ROW_POSITION(mw, i) = y;
+}
+
+void
 xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width,
 		Boolean compute_height)
-#else
-xbaeComputeSize(mw, compute_width, compute_height)
-XbaeMatrixWidget mw;
-Boolean compute_width;
-Boolean compute_height;
-#endif
 {
     unsigned long full_width = NON_FIXED_TOTAL_WIDTH(mw) +
 	FIXED_COLUMN_WIDTH(mw) + TRAILING_FIXED_COLUMN_WIDTH(mw) +
 	ROW_LABEL_WIDTH(mw) + 2 * mw->manager.shadow_thickness;
-    unsigned long full_height = CELL_TOTAL_HEIGHT(mw) +
+    unsigned long full_height = xbaeCellTotalHeight(mw) +
 	FIXED_ROW_HEIGHT(mw) + TRAILING_FIXED_ROW_HEIGHT(mw) +
 	COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
     unsigned long width, height;
@@ -618,8 +597,7 @@ Boolean compute_height;
  * Return the length of the longest row label
  */
 short
-xbaeMaxRowLabel(mw)
-XbaeMatrixWidget mw;
+xbaeMaxRowLabel(XbaeMatrixWidget mw)
 {
     int i;
     short max = 0, len;
@@ -637,9 +615,7 @@ XbaeMatrixWidget mw;
 }
 
 void
-xbaeParseColumnLabel(label, lines)
-String label;
-ColumnLabelLines lines;
+xbaeParseColumnLabel(String label, ColumnLabelLines lines)
 {
     char *nl;
 
@@ -697,11 +673,7 @@ ColumnLabelLines lines;
  * may be useful nevertheless.
  */
 Boolean
-xbaeXYToRowCol(mw, x, y, row, column, cell)
-XbaeMatrixWidget mw;
-int *x, *y;
-int *row, *column;
-CellType cell;
+xbaeXYToRowCol(XbaeMatrixWidget mw, int *x, int *y, int *row, int *column, CellType cell)
 {
     Rectangle rect;
     unsigned int inBox = CLIP_NONE;
@@ -1105,7 +1077,7 @@ CellType cell;
 	    *row = YtoRow(mw, *y);
 	    *row += (CLIP_FIXED_ROWS & inBox ? 0 : mw->matrix.top_row);
 	    *row += ((CLIP_TRAILING_FIXED_ROWS & inBox) ?
-		     (CELL_TOTAL_HEIGHT(mw) - VISIBLE_HEIGHT(mw)) /
+		     (xbaeCellTotalHeight(mw) - VISIBLE_HEIGHT(mw)) /
 		     row_height - mw->matrix.top_row: 0);
 	    	    
 	    *column = -1;
@@ -1273,7 +1245,7 @@ CellType *cell;
 	*x -= FIXED_COLUMN_LABEL_OFFSET(mw);
 	*y -= FIXED_ROW_LABEL_OFFSET(mw);
     }
-    else if (event->xbutton.window == XtWindow(mw))
+    else if (event->xbutton.window == XtWindow(mw)) {
 	if (*x < (int)COLUMN_LABEL_OFFSET(mw) &&
 	    *x > (int)VERT_SB_OFFSET(mw))
 	    *cell = RowLabelCell;
@@ -1282,6 +1254,7 @@ CellType *cell;
 	    *cell = ColumnLabelCell;
 	else
 	    *cell = FixedCell;
+    }
     else if (event->xbutton.window == XtWindow(ClipChild(mw)))
 	*cell = NonFixedCell;
     else if (event->xbutton.window == XtWindow(TextChild(mw)))
@@ -1335,6 +1308,24 @@ int x;
     return i - 1;
 }
 
+int
+xbaeYtoRow(XbaeMatrixWidget mw, int y)
+{
+    int i;
+
+    for (i = 0; i < mw->matrix.rows; i++)
+	if (ROW_POSITION(mw, i) > y)
+	    return i - 1;
+    /*
+     * I have seen cases where this function returned mw->matrix.columns
+     * causing a crash as array bounds are read - AL
+     */
+    if (i > mw->matrix.rows)
+	return mw->matrix.rows - 1;
+
+    return i - 1;
+}
+
 /*
  * Convert a pixel position to the trailing
  * fixed column it is contained in
@@ -1698,3 +1689,37 @@ int *height;
 		ClipChild(mw)->core.y - *ay;
     }
 }
+
+/*
+ * Below are some functions to deal with a multi-threaded environment.
+ * Use these instead of the Xt stuff, to ensure that we can still cope
+ * with X11r5 where this stuff didn't exist.
+ */
+#ifdef  XtSpecificationRelease
+#if     XtSpecificationRelease > 5
+#define R6plus
+#endif
+#endif
+
+void xbaeObjectLock(Widget w)
+{
+#ifdef  R6plus
+        if (XmIsGadget(w))
+                XtAppLock(XtWidgetToApplicationContext(
+                        XtParent(w)));
+        else
+                XtAppLock(XtWidgetToApplicationContext(w));
+#endif
+}
+
+void
+xbaeObjectUnlock(Widget w)
+{
+#ifdef  R6plus
+        if (XmIsGadget(w))
+                XtAppUnlock(XtWidgetToApplicationContext(
+                        XtParent(w)));
+        else
+                XtAppUnlock(XtWidgetToApplicationContext(w));
+#endif
+}
